"""
高级浏览器操作
更复杂的自动化场景
"""
import asyncio
from typing import Optional, List, Dict, Any, Callable
from playwright.async_api import Page, Locator
from loguru import logger
from dataclasses import dataclass
from datetime import datetime


@dataclass
class DownloadTask:
    """下载任务"""
    url: str
    save_path: str
    status: str = "pending"
    progress: float = 0
    error: Optional[str] = None


class BatchOperations:
    """
    批量操作
    支持批量截图、下载、填充等
    """
    
    def __init__(self, page: Page):
        self.page = page
    
    async def batch_screenshot(
        self,
        urls: List[str],
        output_dir: str = "./screenshots",
        naming: str = "url",
        wait_after_load: float = 1.0,
        progress_callback: Optional[Callable[[int, int], None]] = None,
    ) -> List[str]:
        """
        批量截图
        
        Args:
            urls: URL列表
            output_dir: 输出目录
            naming: 命名方式 (url, index, timestamp)
            wait_after_load: 加载后等待时间
            
        Returns:
            List[str]: 截图路径列表
        """
        from pathlib import Path
        from urllib.parse import urlparse
        
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        paths = []
        
        for i, url in enumerate(urls):
            try:
                await self.page.goto(url)
                await asyncio.sleep(wait_after_load)
                
                # 生成文件名
                if naming == "url":
                    parsed = urlparse(url)
                    name = parsed.netloc + parsed.path.replace("/", "_")
                elif naming == "index":
                    name = f"screenshot_{i:04d}"
                else:
                    name = datetime.now().strftime("%Y%m%d_%H%M%S")
                
                path = str(Path(output_dir) / f"{name}.png")
                await self.page.screenshot(path=path)
                paths.append(path)
                
                logger.info(f"截图完成: {url}")
                
                if progress_callback:
                    progress_callback(i + 1, len(urls))
                    
            except Exception as e:
                logger.error(f"截图失败: {url}, error={e}")
                paths.append(None)
        
        return paths
    
    async def batch_fill(
        self,
        form_data: List[Dict[str, str]],
        form_selector: str = "form",
        submit_selector: Optional[str] = None,
        delay: float = 1.0,
    ) -> List[bool]:
        """
        批量填充表单
        
        Args:
            form_data: 表单数据列表
            form_selector: 表单选择器
            submit_selector: 提交按钮选择器
            delay: 填充间隔
            
        Returns:
            List[bool]: 成功状态列表
        """
        from .form import FormHandler
        
        form_handler = FormHandler(self.page)
        results = []
        
        for i, data in enumerate(form_data):
            try:
                await form_handler.fill(data, form_selector)
                
                if submit_selector:
                    await self.page.click(submit_selector)
                    await asyncio.sleep(delay)
                
                results.append(True)
                logger.info(f"表单填充成功: {i + 1}/{len(form_data)}")
                
            except Exception as e:
                logger.error(f"表单填充失败: {e}")
                results.append(False)
        
        return results
    
    async def batch_download(
        self,
        urls: List[str],
        output_dir: str = "./downloads",
        progress_callback: Optional[Callable[[int, int], None]] = None,
    ) -> List[DownloadTask]:
        """
        批量下载文件
        
        Args:
            urls: 下载URL列表
            output_dir: 输出目录
            progress_callback: 进度回调
            
        Returns:
            List[DownloadTask]: 下载任务列表
        """
        from pathlib import Path
        
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        tasks = []
        
        for i, url in enumerate(urls):
            task = DownloadTask(url=url, save_path="")
            tasks.append(task)
            
            try:
                async with self.page.expect_download() as download_info:
                    # 尝试直接导航到下载链接
                    await self.page.evaluate(f"window.location.href = '{url}'")
                
                download = await download_info.value
                save_path = str(Path(output_dir) / download.suggested_filename)
                await download.save_as(save_path)
                
                task.save_path = save_path
                task.status = "completed"
                
                logger.info(f"下载完成: {url}")
                
            except Exception as e:
                task.status = "failed"
                task.error = str(e)
                logger.error(f"下载失败: {url}, error={e}")
            
            if progress_callback:
                progress_callback(i + 1, len(urls))
        
        return tasks


class AutomationFlow:
    """
    自动化流程
    定义和执行自动化工作流
    """
    
    def __init__(self, page: Page):
        self.page = page
        self.steps: List[Dict[str, Any]] = []
        self._current_step = 0
    
    def add_step(
        self,
        action: str,
        selector: Optional[str] = None,
        value: Optional[str] = None,
        wait: Optional[float] = None,
        condition: Optional[Callable] = None,
    ) -> "AutomationFlow":
        """
        添加步骤
        
        Args:
            action: 动作类型
            selector: 选择器
            value: 值
            wait: 等待时间
            condition: 条件函数
            
        Returns:
            AutomationFlow: self (链式调用)
        """
        self.steps.append({
            "action": action,
            "selector": selector,
            "value": value,
            "wait": wait,
            "condition": condition,
        })
        return self
    
    def navigate(self, url: str) -> "AutomationFlow":
        """导航步骤"""
        return self.add_step("navigate", value=url)
    
    def click(self, selector: str, wait: float = None) -> "AutomationFlow":
        """点击步骤"""
        return self.add_step("click", selector=selector, wait=wait)
    
    def fill(self, selector: str, value: str) -> "AutomationFlow":
        """填充步骤"""
        return self.add_step("fill", selector=selector, value=value)
    
    def wait(self, seconds: float) -> "AutomationFlow":
        """等待步骤"""
        return self.add_step("wait", value=str(seconds))
    
    def wait_for_selector(self, selector: str, timeout: float = 30) -> "AutomationFlow":
        """等待元素步骤"""
        return self.add_step("wait_for_selector", selector=selector, value=str(timeout))
    
    def scroll(self, distance: int = 500) -> "AutomationFlow":
        """滚动步骤"""
        return self.add_step("scroll", value=str(distance))
    
    def screenshot(self, path: str) -> "AutomationFlow":
        """截图步骤"""
        return self.add_step("screenshot", value=path)
    
    def condition(self, check_func: Callable) -> "AutomationFlow":
        """条件步骤"""
        return self.add_step("condition", condition=check_func)
    
    async def run(
        self,
        on_step_start: Optional[Callable[[int, Dict], None]] = None,
        on_step_end: Optional[Callable[[int, Dict, bool], None]] = None,
    ) -> bool:
        """
        执行流程
        
        Args:
            on_step_start: 步骤开始回调
            on_step_end: 步骤结束回调
            
        Returns:
            bool: 是否全部成功
        """
        logger.info(f"开始执行流程: {len(self.steps)} 步")
        
        for i, step in enumerate(self.steps):
            self._current_step = i
            
            if on_step_start:
                on_step_start(i, step)
            
            success = await self._execute_step(step)
            
            if on_step_end:
                on_step_end(i, step, success)
            
            if not success and step.get("required", True):
                logger.error(f"流程中断: 步骤 {i + 1} 失败")
                return False
        
        logger.info("流程执行完成")
        return True
    
    async def _execute_step(self, step: Dict) -> bool:
        """执行单个步骤"""
        try:
            action = step["action"]
            
            if action == "navigate":
                await self.page.goto(step["value"])
            
            elif action == "click":
                await self.page.click(step["selector"])
                if step.get("wait"):
                    await asyncio.sleep(float(step["wait"]))
            
            elif action == "fill":
                await self.page.fill(step["selector"], step["value"])
            
            elif action == "wait":
                await asyncio.sleep(float(step["value"]))
            
            elif action == "wait_for_selector":
                await self.page.wait_for_selector(
                    step["selector"],
                    timeout=float(step["value"]) * 1000
                )
            
            elif action == "scroll":
                await self.page.mouse.wheel(0, int(step["value"]))
            
            elif action == "screenshot":
                await self.page.screenshot(path=step["value"])
            
            elif action == "condition":
                if asyncio.iscoroutinefunction(step["condition"]):
                    result = await step["condition"]()
                else:
                    result = step["condition"]()
                
                if not result:
                    return False
            
            logger.debug(f"步骤完成: {action}")
            return True
            
        except Exception as e:
            logger.error(f"步骤执行失败: {e}")
            return False
    
    def save(self, filepath: str):
        """保存流程"""
        import json
        
        data = {
            "steps": self.steps,
            "saved_at": datetime.now().isoformat(),
        }
        
        with open(filepath, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
        
        logger.info(f"流程已保存: {filepath}")
    
    @classmethod
    def load(cls, page: Page, filepath: str) -> "AutomationFlow":
        """加载流程"""
        import json
        
        with open(filepath, 'r', encoding='utf-8') as f:
            data = json.load(f)
        
        flow = cls(page)
        flow.steps = data["steps"]
        
        logger.info(f"流程已加载: {filepath}")
        return flow


class TabManager:
    """
    标签页管理器
    高级多标签页操作
    """
    
    def __init__(self, page: Page):
        self.page = page
        self.context = page.context
        self._managed_tabs: Dict[str, Page] = {}
    
    async def open_tab(self, name: str, url: str = "about:blank") -> Page:
        """打开命名标签页"""
        new_page = await self.context.new_page()
        await new_page.goto(url)
        self._managed_tabs[name] = new_page
        logger.info(f"打开标签页: {name}")
        return new_page
    
    async def switch_to(self, name: str) -> Page:
        """切换到标签页"""
        if name not in self._managed_tabs:
            raise ValueError(f"标签页不存在: {name}")
        return self._managed_tabs[name]
    
    async def close_tab(self, name: str):
        """关闭标签页"""
        if name in self._managed_tabs:
            await self._managed_tabs[name].close()
            del self._managed_tabs[name]
            logger.info(f"关闭标签页: {name}")
    
    async def broadcast(self, action: Callable[[Page], Any]):
        """在所有标签页执行操作"""
        results = {}
        for name, page in self._managed_tabs.items():
            try:
                if asyncio.iscoroutinefunction(action):
                    results[name] = await action(page)
                else:
                    results[name] = action(page)
            except Exception as e:
                results[name] = {"error": str(e)}
        return results
    
    async def screenshot_all(self, output_dir: str = "./tabs"):
        """截取所有标签页"""
        from pathlib import Path
        
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        
        for name, page in self._managed_tabs.items():
            try:
                path = str(Path(output_dir) / f"{name}.png")
                await page.screenshot(path=path)
                logger.info(f"截图: {name} -> {path}")
            except Exception as e:
                logger.error(f"截图失败: {name}, error={e}")


class SessionRecorder:
    """
    会话录制器
    录制整个浏览会话
    """
    
    def __init__(self, page: Page):
        self.page = page
        self._events: List[Dict] = []
        self._is_recording = False
        self._start_time: Optional[datetime] = None
    
    def start(self):
        """开始录制"""
        self._is_recording = True
        self._start_time = datetime.now()
        self._events.clear()
        
        # 监听页面事件
        self.page.on("load", self._on_load)
        self.page.on("console", self._on_console)
        self.page.on("pageerror", self._on_error)
        
        logger.info("会话录制开始")
    
    def stop(self):
        """停止录制"""
        self._is_recording = False
        
        self.page.remove_listener("load", self._on_load)
        self.page.remove_listener("console", self._on_console)
        self.page.remove_listener("pageerror", self._on_error)
        
        logger.info(f"会话录制结束: {len(self._events)} 个事件")
    
    def _record_event(self, event_type: str, data: Any):
        """记录事件"""
        if not self._is_recording:
            return
        
        self._events.append({
            "type": event_type,
            "data": data,
            "timestamp": datetime.now().isoformat(),
            "url": self.page.url,
        })
    
    def _on_load(self):
        self._record_event("load", {"url": self.page.url})
    
    def _on_console(self, msg):
        self._record_event("console", {"type": msg.type, "text": msg.text})
    
    def _on_error(self, error):
        self._record_event("error", {"message": str(error)})
    
    def save(self, filepath: str):
        """保存录制"""
        import json
        
        data = {
            "events": self._events,
            "start_time": self._start_time.isoformat() if self._start_time else None,
            "end_time": datetime.now().isoformat(),
            "duration_seconds": (datetime.now() - self._start_time).total_seconds() if self._start_time else 0,
        }
        
        with open(filepath, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
        
        logger.info(f"会话录制已保存: {filepath}")
    
    def get_summary(self) -> Dict:
        """获取摘要"""
        summary = {
            "total_events": len(self._events),
            "errors": sum(1 for e in self._events if e["type"] == "error"),
            "consoles": sum(1 for e in self._events if e["type"] == "console"),
            "pages_loaded": sum(1 for e in self._events if e["type"] == "load"),
        }
        return summary



"""
HTTP Header 安全检查
检查 CSP、HSTS、X-Frame-Options 等安全头
"""
from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any
from datetime import datetime
from playwright.async_api import Page, Response
from loguru import logger


@dataclass
class HeaderCheckConfig:
    """Header 检查配置"""
    check_csp: bool = True
    check_hsts: bool = True
    check_frame_options: bool = True
    check_content_type_options: bool = True
    check_xss_protection: bool = True
    check_referrer_policy: bool = True
    check_permissions_policy: bool = True
    strict_mode: bool = False


@dataclass
class HeaderIssue:
    """Header 问题"""
    severity: str  # high, medium, low, info
    header: str
    current_value: Optional[str]
    issue: str
    recommendation: str
    reference: Optional[str] = None


@dataclass
class HeaderCheckResult:
    """Header 检查结果"""
    url: str
    headers: Dict[str, str] = field(default_factory=dict)
    issues: List[HeaderIssue] = field(default_factory=list)
    score: int = 100  # 安全评分 0-100
    timestamp: str = field(default_factory=lambda: datetime.now().isoformat())

    @property
    def has_issues(self) -> bool:
        return len(self.issues) > 0

    @property
    def high_severity_count(self) -> int:
        return sum(1 for i in self.issues if i.severity == "high")

    @property
    def medium_severity_count(self) -> int:
        return sum(1 for i in self.issues if i.severity == "medium")


class HeaderChecker:
    """
    HTTP Header 安全检查器
    检查常见安全头配置
    """

    # 推荐的安全 Headers
    RECOMMENDED_HEADERS = {
        "content-security-policy": {
            "severity": "high",
            "description": "内容安全策略，防止 XSS 和数据注入",
            "recommendation": "设置严格的 CSP，限制脚本来源",
        },
        "strict-transport-security": {
            "severity": "high",
            "description": "HSTS，强制使用 HTTPS",
            "recommendation": "设置 max-age 至少 31536000 秒",
        },
        "x-frame-options": {
            "severity": "medium",
            "description": "防止点击劫持",
            "recommendation": "设置为 DENY 或 SAMEORIGIN",
        },
        "x-content-type-options": {
            "severity": "medium",
            "description": "防止 MIME 类型嗅探",
            "recommendation": "设置为 nosniff",
        },
        "x-xss-protection": {
            "severity": "low",
            "description": "XSS 过滤器（已弃用但仍建议设置）",
            "recommendation": "设置为 0 以禁用（现代浏览器已移除此功能）",
        },
        "referrer-policy": {
            "severity": "medium",
            "description": "控制 Referrer 信息",
            "recommendation": "设置为 strict-origin-when-cross-origin",
        },
        "permissions-policy": {
            "severity": "medium",
            "description": "控制浏览器功能权限",
            "recommendation": "限制摄像头、麦克风等敏感功能",
        },
    }

    def __init__(
        self,
        page: Page,
        config: Optional[HeaderCheckConfig] = None,
    ):
        self.page = page
        self.config = config or HeaderCheckConfig()

    # ==================== 检查方法 ====================

    async def check(self, url: Optional[str] = None) -> HeaderCheckResult:
        """
        执行 Header 安全检查

        Args:
            url: 要检查的 URL

        Returns:
            HeaderCheckResult: 检查结果
        """
        target_url = url or self.page.url
        result = HeaderCheckResult(url=target_url)

        # 获取响应头
        headers = await self._fetch_headers(target_url)
        result.headers = headers

        # 检查各个安全头
        if self.config.check_csp:
            self._check_csp(headers, result)

        if self.config.check_hsts:
            self._check_hsts(headers, result)

        if self.config.check_frame_options:
            self._check_frame_options(headers, result)

        if self.config.check_content_type_options:
            self._check_content_type_options(headers, result)

        if self.config.check_xss_protection:
            self._check_xss_protection(headers, result)

        if self.config.check_referrer_policy:
            self._check_referrer_policy(headers, result)

        if self.config.check_permissions_policy:
            self._check_permissions_policy(headers, result)

        # 计算安全评分
        result.score = self._calculate_score(result)

        logger.info(
            f"Header 检查完成: {target_url}, "
            f"评分: {result.score}, 问题: {len(result.issues)}"
        )

        return result

    async def check_multiple_urls(self, urls: List[str]) -> Dict[str, HeaderCheckResult]:
        """
        检查多个 URL

        Args:
            urls: URL 列表

        Returns:
            Dict[str, HeaderCheckResult]: 结果映射
        """
        results = {}

        for url in urls:
            try:
                result = await self.check(url)
                results[url] = result
            except Exception as e:
                logger.error(f"检查失败: {url}, error={e}")

        return results

    # ==================== 具体检查 ====================

    def _check_csp(self, headers: Dict[str, str], result: HeaderCheckResult):
        """检查 CSP"""
        csp = self._get_header(headers, "content-security-policy")

        if not csp:
            result.issues.append(HeaderIssue(
                severity="high",
                header="Content-Security-Policy",
                current_value=None,
                issue="缺少 Content-Security-Policy 头",
                recommendation="添加 CSP 头以防止 XSS 和注入攻击",
                reference="https://developer.mozilla.org/en-US/docs/Web/HTTP/CSP",
            ))
            return

        # 检查 CSP 配置
        csp_issues = self._analyze_csp(csp)
        for issue in csp_issues:
            result.issues.append(HeaderIssue(
                severity=issue.get("severity", "medium"),
                header="Content-Security-Policy",
                current_value=csp,
                issue=issue.get("issue", ""),
                recommendation=issue.get("recommendation", ""),
            ))

    def _check_hsts(self, headers: Dict[str, str], result: HeaderCheckResult):
        """检查 HSTS"""
        hsts = self._get_header(headers, "strict-transport-security")

        if not hsts:
            result.issues.append(HeaderIssue(
                severity="high",
                header="Strict-Transport-Security",
                current_value=None,
                issue="缺少 HSTS 头",
                recommendation="添加 HSTS 头以强制使用 HTTPS",
            ))
            return

        # 检查 max-age
        if "max-age" in hsts:
            max_age = self._extract_max_age(hsts)
            if max_age and max_age < 31536000:  # 少于一年
                result.issues.append(HeaderIssue(
                    severity="medium",
                    header="Strict-Transport-Security",
                    current_value=hsts,
                    issue="max-age 值过小",
                    recommendation="将 max-age 设置为至少 31536000 秒（一年）",
                ))

        # 检查 includeSubDomains
        if "includeSubDomains" not in hsts and self.config.strict_mode:
            result.issues.append(HeaderIssue(
                severity="low",
                header="Strict-Transport-Security",
                current_value=hsts,
                issue="未包含 includeSubDomains 指令",
                recommendation="添加 includeSubDomains 以保护子域名",
            ))

    def _check_frame_options(self, headers: Dict[str, str], result: HeaderCheckResult):
        """检查 X-Frame-Options"""
        xfo = self._get_header(headers, "x-frame-options")

        if not xfo:
            result.issues.append(HeaderIssue(
                severity="medium",
                header="X-Frame-Options",
                current_value=None,
                issue="缺少 X-Frame-Options 头",
                recommendation="添加 X-Frame-Options 防止点击劫持",
            ))
            return

        if xfo.upper() not in ["DENY", "SAMEORIGIN"]:
            result.issues.append(HeaderIssue(
                severity="medium",
                header="X-Frame-Options",
                current_value=xfo,
                issue="X-Frame-Options 值不安全",
                recommendation="设置为 DENY 或 SAMEORIGIN",
            ))

    def _check_content_type_options(self, headers: Dict[str, str], result: HeaderCheckResult):
        """检查 X-Content-Type-Options"""
        xcto = self._get_header(headers, "x-content-type-options")

        if not xcto:
            result.issues.append(HeaderIssue(
                severity="medium",
                header="X-Content-Type-Options",
                current_value=None,
                issue="缺少 X-Content-Type-Options 头",
                recommendation="添加 X-Content-Type-Options: nosniff",
            ))
            return

        if xcto.lower() != "nosniff":
            result.issues.append(HeaderIssue(
                severity="low",
                header="X-Content-Type-Options",
                current_value=xcto,
                issue="X-Content-Type-Options 值不正确",
                recommendation="设置为 nosniff",
            ))

    def _check_xss_protection(self, headers: Dict[str, str], result: HeaderCheckResult):
        """检查 X-XSS-Protection"""
        xss = self._get_header(headers, "x-xss-protection")

        # X-XSS-Protection 已弃用，但仍检查
        if xss and "1" in xss and "mode=block" not in xss:
            result.issues.append(HeaderIssue(
                severity="info",
                header="X-XSS-Protection",
                current_value=xss,
                issue="X-XSS-Protection 应使用 block 模式",
                recommendation="设置为 0 以禁用（现代浏览器已弃用此功能）",
            ))

    def _check_referrer_policy(self, headers: Dict[str, str], result: HeaderCheckResult):
        """检查 Referrer-Policy"""
        rp = self._get_header(headers, "referrer-policy")

        if not rp:
            result.issues.append(HeaderIssue(
                severity="medium",
                header="Referrer-Policy",
                current_value=None,
                issue="缺少 Referrer-Policy 头",
                recommendation="添加 Referrer-Policy 控制信息泄露",
            ))
            return

        safe_values = [
            "no-referrer",
            "no-referrer-when-downgrade",
            "strict-origin",
            "strict-origin-when-cross-origin",
            "same-origin",
        ]

        if rp.lower() not in safe_values:
            result.issues.append(HeaderIssue(
                severity="low",
                header="Referrer-Policy",
                current_value=rp,
                issue="Referrer-Policy 值可能不够安全",
                recommendation="使用 strict-origin-when-cross-origin 或更严格的策略",
            ))

    def _check_permissions_policy(self, headers: Dict[str, str], result: HeaderCheckResult):
        """检查 Permissions-Policy"""
        pp = self._get_header(headers, "permissions-policy")

        if not pp and self.config.strict_mode:
            result.issues.append(HeaderIssue(
                severity="low",
                header="Permissions-Policy",
                current_value=None,
                issue="缺少 Permissions-Policy 头",
                recommendation="添加 Permissions-Policy 限制敏感功能",
            ))

    # ==================== 辅助方法 ====================

    async def _fetch_headers(self, url: str) -> Dict[str, str]:
        """获取响应头"""
        try:
            response = await self.page.goto(url, wait_until="domcontentloaded")
            if response:
                return dict(response.headers)
        except Exception as e:
            logger.error(f"获取响应头失败: {url}, error={e}")
        return {}

    def _get_header(self, headers: Dict[str, str], name: str) -> Optional[str]:
        """获取指定 Header（不区分大小写）"""
        for key, value in headers.items():
            if key.lower() == name:
                return value
        return None

    def _analyze_csp(self, csp: str) -> List[Dict[str, str]]:
        """分析 CSP 配置"""
        issues = []

        # 检查 unsafe-inline
        if "unsafe-inline" in csp:
            issues.append({
                "severity": "high",
                "issue": "CSP 包含 unsafe-inline，降低安全性",
                "recommendation": "移除 unsafe-inline，使用 nonce 或 hash",
            })

        # 检查 unsafe-eval
        if "unsafe-eval" in csp:
            issues.append({
                "severity": "high",
                "issue": "CSP 包含 unsafe-eval，允许 eval 执行",
                "recommendation": "移除 unsafe-eval",
            })

        # 检查 data: URI
        if "data:" in csp:
            issues.append({
                "severity": "medium",
                "issue": "CSP 允许 data: URI",
                "recommendation": "移除 data: 源",
            })

        # 检查 * 通配符
        if "*" in csp and "default-src" in csp:
            issues.append({
                "severity": "medium",
                "issue": "CSP 使用通配符源",
                "recommendation": "限制为具体的域名",
            })

        return issues

    def _extract_max_age(self, hsts: str) -> Optional[int]:
        """提取 HSTS max-age 值"""
        import re
        match = re.search(r"max-age=(\d+)", hsts)
        if match:
            return int(match.group(1))
        return None

    def _calculate_score(self, result: HeaderCheckResult) -> int:
        """计算安全评分"""
        score = 100

        for issue in result.issues:
            if issue.severity == "high":
                score -= 20
            elif issue.severity == "medium":
                score -= 10
            elif issue.severity == "low":
                score -= 5

        return max(0, score)

    # ==================== 便捷方法 ====================

    async def quick_check(self) -> bool:
        """
        快速检查，返回是否安全

        Returns:
            bool: 是否通过检查（无高危问题）
        """
        result = await self.check()
        return result.high_severity_count == 0

    async def get_security_score(self, url: Optional[str] = None) -> int:
        """
        获取安全评分

        Args:
            url: 要检查的 URL

        Returns:
            int: 安全评分 0-100
        """
        result = await self.check(url)
        return result.score


def create_header_checker(
    page: Page,
    config: Optional[HeaderCheckConfig] = None,
) -> HeaderChecker:
    """创建 Header 检查器实例"""
    return HeaderChecker(page, config)



"""
文件管理器
文件上传/下载统一管理，支持进度监控
"""
import os
import mimetypes
from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any
from pathlib import Path
from playwright.async_api import Page, Download, FileChooser
from loguru import logger


@dataclass
class FileResult:
    """文件操作结果"""
    success: bool
    path: str
    size: int = 0
    error: Optional[str] = None


@dataclass
class UploadConfig:
    """上传配置"""
    selector: str
    file_paths: List[str]


@dataclass
class DownloadConfig:
    """下载配置"""
    trigger_selector: str
    save_dir: str = "./downloads"
    file_name: Optional[str] = None


class FileManager:
    """
    文件管理器
    统一管理文件上传/下载操作
    """

    def __init__(self, page: Page, download_dir: str = "./downloads"):
        self.page = page
        self.download_dir = download_dir
        self._active_downloads: Dict[str, Download] = {}
        self._ensure_dir(self.download_dir)

    # ==================== 文件上传 ====================

    async def upload_file(
        self,
        selector: str,
        file_path: str,
    ) -> FileResult:
        """
        上传单个文件

        Args:
            selector: 文件输入框选择器
            file_path: 文件路径

        Returns:
            FileResult: 上传结果
        """
        try:
            # 检查文件是否存在
            if not os.path.exists(file_path):
                return FileResult(
                    success=False,
                    path=file_path,
                    error=f"文件不存在: {file_path}",
                )

            # 尝试直接设置文件
            input_locator = self.page.locator(f'{selector} input[type="file"]')
            input_count = await input_locator.count()

            if input_count > 0:
                await input_locator.set_input_files(file_path)
            else:
                # 尝试直接选择器
                direct_locator = self.page.locator(selector)
                is_file_input = await direct_locator.evaluate(
                    "el => el.tagName === 'INPUT' && el.type === 'file'"
                )

                if is_file_input:
                    await direct_locator.set_input_files(file_path)
                else:
                    # 使用 file chooser
                    async with self.page.expect_file_chooser() as fc_info:
                        await direct_locator.click()

                    file_chooser = await fc_info.value
                    await file_chooser.set_files(file_path)

            file_size = os.path.getsize(file_path)
            logger.debug(f"文件上传成功: {file_path} ({file_size} bytes)")

            return FileResult(
                success=True,
                path=file_path,
                size=file_size,
            )

        except Exception as e:
            logger.error(f"文件上传失败: {file_path}, error={e}")
            return FileResult(
                success=False,
                path=file_path,
                error=str(e),
            )

    async def upload_files(
        self,
        selector: str,
        file_paths: List[str],
    ) -> List[FileResult]:
        """
        上传多个文件

        Args:
            selector: 文件输入框选择器
            file_paths: 文件路径列表

        Returns:
            List[FileResult]: 上传结果列表
        """
        results: List[FileResult] = []

        # 检查是否支持多文件
        input_locator = self.page.locator(f'{selector} input[type="file"]')
        input_count = await input_locator.count()

        if input_count > 0:
            is_multiple = await input_locator.evaluate(
                "el => el.multiple"
            )

            if is_multiple:
                # 一次性上传所有文件
                existing_files = [f for f in file_paths if os.path.exists(f)]
                if existing_files:
                    await input_locator.set_input_files(existing_files)
                    for f in existing_files:
                        results.append(FileResult(
                            success=True,
                            path=f,
                            size=os.path.getsize(f),
                        ))
                    for f in file_paths:
                        if f not in existing_files:
                            results.append(FileResult(
                                success=False,
                                path=f,
                                error="文件不存在",
                            ))
                    return results

        # 逐个上传
        for file_path in file_paths:
            result = await self.upload_file(selector, file_path)
            results.append(result)

        return results

    async def upload(self, config: UploadConfig) -> List[FileResult]:
        """
        使用配置对象上传文件

        Args:
            config: 上传配置

        Returns:
            List[FileResult]: 上传结果列表
        """
        return await self.upload_files(config.selector, config.file_paths)

    async def upload_by_drop(
        self,
        selector: str,
        file_path: str,
    ) -> FileResult:
        """
        通过拖放上传文件

        Args:
            selector: 放置区域选择器
            file_path: 文件路径

        Returns:
            FileResult: 上传结果
        """
        try:
            if not os.path.exists(file_path):
                return FileResult(
                    success=False,
                    path=file_path,
                    error=f"文件不存在: {file_path}",
                )

            drop_zone = self.page.locator(selector)
            file_name = os.path.basename(file_path)
            file_size = os.path.getsize(file_path)
            mime_type = self._get_mime_type(file_path)

            # 模拟拖放事件
            await drop_zone.dispatch_event("drop", {
                "dataTransfer": {
                    "files": [{
                        "name": file_name,
                        "size": file_size,
                        "type": mime_type,
                    }],
                },
            })

            logger.debug(f"拖放上传完成: {file_path}")

            return FileResult(
                success=True,
                path=file_path,
                size=file_size,
            )

        except Exception as e:
            logger.error(f"拖放上传失败: {file_path}, error={e}")
            return FileResult(
                success=False,
                path=file_path,
                error=str(e),
            )

    # ==================== 文件下载 ====================

    async def download_file(
        self,
        trigger_selector: str,
        save_path: Optional[str] = None,
    ) -> FileResult:
        """
        下载文件

        Args:
            trigger_selector: 触发下载的元素选择器
            save_path: 保存路径

        Returns:
            FileResult: 下载结果
        """
        try:
            # 监听下载事件并点击触发
            async with self.page.expect_download() as download_info:
                await self.page.locator(trigger_selector).click()

            download = await download_info.value

            # 确定保存路径
            suggested_name = download.suggested_filename
            final_path = save_path or os.path.join(self.download_dir, suggested_name)

            # 确保目录存在
            self._ensure_dir(os.path.dirname(final_path))

            # 保存文件
            await download.save_as(final_path)

            file_size = os.path.getsize(final_path)
            self._active_downloads.pop(suggested_name, None)

            logger.debug(f"文件下载成功: {final_path} ({file_size} bytes)")

            return FileResult(
                success=True,
                path=final_path,
                size=file_size,
            )

        except Exception as e:
            logger.error(f"文件下载失败: {e}")
            return FileResult(
                success=False,
                path=save_path or "unknown",
                error=str(e),
            )

    async def download(self, config: DownloadConfig) -> FileResult:
        """
        使用配置对象下载文件

        Args:
            config: 下载配置

        Returns:
            FileResult: 下载结果
        """
        save_path = None
        if config.file_name:
            save_path = os.path.join(config.save_dir, config.file_name)

        self._ensure_dir(config.save_dir)
        return await self.download_file(config.trigger_selector, save_path)

    async def wait_for_download(self, timeout: float = 60000) -> Optional[Download]:
        """
        等待下载开始

        Args:
            timeout: 超时时间（毫秒）

        Returns:
            Optional[Download]: 下载对象
        """
        try:
            async with self.page.expect_download(timeout=timeout) as download_info:
                pass
            return await download_info.value
        except Exception:
            return None

    async def cancel_download(self, download: Download) -> None:
        """取消下载"""
        await download.cancel()

    async def get_download_progress(self, download: Download) -> int:
        """
        获取下载进度

        Args:
            download: 下载对象

        Returns:
            int: 已下载字节数
        """
        try:
            path = await download.path()
            if path and os.path.exists(path):
                return os.path.getsize(path)
        except Exception:
            pass
        return 0

    # ==================== 文件信息 ====================

    def file_exists(self, file_path: str) -> bool:
        """检查文件是否存在"""
        return os.path.exists(file_path)

    def get_file_size(self, file_path: str) -> int:
        """获取文件大小"""
        try:
            return os.path.getsize(file_path)
        except Exception:
            return 0

    def get_file_info(self, file_path: str) -> Dict[str, Any]:
        """
        获取文件信息

        Args:
            file_path: 文件路径

        Returns:
            dict: 文件信息
        """
        try:
            stat = os.stat(file_path)
            return {
                "exists": True,
                "size": stat.st_size,
                "created": stat.st_ctime,
                "modified": stat.st_mtime,
                "extension": os.path.splitext(file_path)[1],
            }
        except Exception:
            return {
                "exists": False,
                "size": 0,
                "created": None,
                "modified": None,
                "extension": "",
            }

    def delete_file(self, file_path: str) -> bool:
        """删除文件"""
        try:
            os.remove(file_path)
            return True
        except Exception:
            return False

    def rename_file(self, old_path: str, new_path: str) -> bool:
        """重命名文件"""
        try:
            os.rename(old_path, new_path)
            return True
        except Exception:
            return False

    def copy_file(self, source: str, destination: str) -> bool:
        """复制文件"""
        try:
            import shutil
            shutil.copy2(source, destination)
            return True
        except Exception:
            return False

    # ==================== 目录操作 ====================

    def create_dir(self, dir_path: str) -> bool:
        """创建目录"""
        try:
            os.makedirs(dir_path, exist_ok=True)
            return True
        except Exception:
            return False

    def remove_dir(self, dir_path: str, recursive: bool = False) -> bool:
        """删除目录"""
        try:
            import shutil
            if recursive:
                shutil.rmtree(dir_path)
            else:
                os.rmdir(dir_path)
            return True
        except Exception:
            return False

    def list_dir(self, dir_path: str) -> List[str]:
        """列出目录内容"""
        try:
            return os.listdir(dir_path)
        except Exception:
            return []

    def clear_download_dir(self) -> bool:
        """清空下载目录"""
        return self.remove_dir(self.download_dir, recursive=True)

    # ==================== 私有方法 ====================

    def _ensure_dir(self, dir_path: str) -> None:
        """确保目录存在"""
        if dir_path and not os.path.exists(dir_path):
            os.makedirs(dir_path, exist_ok=True)

    def _get_mime_type(self, file_path: str) -> str:
        """获取 MIME 类型"""
        mime_type, _ = mimetypes.guess_type(file_path)
        return mime_type or "application/octet-stream"


def create_file_manager(page: Page, download_dir: str = "./downloads") -> FileManager:
    """创建文件管理器实例"""
    return FileManager(page, download_dir)



"""
浏览器实例池
管理多个浏览器实例
"""
import asyncio
from typing import Optional, Dict, Any, Callable, Awaitable, TypeVar
from loguru import logger

from .types import BrowserPoolConfig, BrowserInstance

T = TypeVar('T')


class BrowserPool:
    """浏览器实例池，管理多个浏览器实例"""

    def __init__(self, config: Optional[BrowserPoolConfig] = None):
        self.config = config or BrowserPoolConfig()
        self._instances: Dict[str, BrowserInstance] = {}
        self._playwright = None
        self._lock = asyncio.Lock()
        self._instance_counter = 0
        self._is_initialized = False

    async def initialize(self):
        if self._is_initialized:
            return
        from playwright.async_api import async_playwright
        self._playwright = await async_playwright().start()
        self._is_initialized = True
        logger.info(f"浏览器池已初始化，最大实例数: {self.config.max_instances}")

    async def close(self):
        for instance_id in list(self._instances.keys()):
            await self._close_instance(instance_id)
        if self._playwright:
            await self._playwright.stop()
            self._playwright = None
        self._is_initialized = False

    async def __aenter__(self):
        await self.initialize()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()

    async def acquire(self) -> BrowserInstance:
        if not self._is_initialized:
            await self.initialize()
        async with self._lock:
            available = self._find_available_instance()
            if available:
                available.is_busy = True
                available.last_used = datetime.now()
                return available
            if len(self._instances) < self.config.max_instances:
                instance = await self._create_instance()
                instance.is_busy = True
                return instance
            raise RuntimeError("浏览器池已满，无可用实例")

    async def release(self, instance_id: str):
        async with self._lock:
            instance = self._instances.get(instance_id)
            if instance:
                instance.is_busy = False
                instance.last_used = datetime.now()

    async def _create_instance(self) -> BrowserInstance:
        from datetime import datetime
        self._instance_counter += 1
        instance_id = f"browser_{self._instance_counter}"
        browser_launcher = getattr(self._playwright, self.config.browser_type)
        browser = await browser_launcher.launch(headless=self.config.headless, timeout=self.config.timeout)
        instance = BrowserInstance(id=instance_id, browser=browser)
        self._instances[instance_id] = instance
        return instance

    async def _close_instance(self, instance_id: str):
        instance = self._instances.pop(instance_id, None)
        if instance:
            for context in instance.contexts:
                await context.close()
            await instance.browser.close()

    def _find_available_instance(self) -> Optional[BrowserInstance]:
        for instance in self._instances.values():
            if not instance.is_busy and instance.context_count < self.config.max_contexts_per_browser:
                return instance
        return None

    async def create_context(self, instance_id: Optional[str] = None, **options) -> Any:
        if instance_id:
            instance = self._instances.get(instance_id)
            if not instance:
                raise ValueError(f"实例不存在: {instance_id}")
        else:
            instance = await self.acquire()
        context = await instance.browser.new_context(**options)
        instance.contexts.append(context)
        return context

    async def execute(self, task: Callable[[Any], Awaitable[T]], **options) -> T:
        instance = await self.acquire()
        context = None
        page = None
        try:
            context = await self.create_context(instance.id, **options)
            page = await context.new_page()
            return await task(page)
        finally:
            if page:
                await page.close()
            if context:
                await context.close()
            await self.release(instance.id)

    def get_stats(self) -> Dict[str, Any]:
        return {
            "total_instances": len(self._instances),
            "busy_instances": sum(1 for i in self._instances.values() if i.is_busy),
            "total_contexts": sum(i.context_count for i in self._instances.values()),
            "is_initialized": self._is_initialized,
        }

    async def scale_to(self, target_count: int):
        """扩缩容到指定实例数"""
        async with self._lock:
            current = len(self._instances)
            if target_count > current:
                for _ in range(target_count - current):
                    await self._create_instance()
            elif target_count < current:
                idle_instances = [
                    i for i in self._instances.values()
                    if not i.is_busy
                ]
                for instance in idle_instances[current - target_count:]:
                    await self._close_instance(instance.id)



"""
调试辅助模块
调试工具、错误处理增强
"""
import traceback
from typing import Optional, Dict, Any, List, Callable
from playwright.async_api import Page, Locator
from loguru import logger
from dataclasses import dataclass
from datetime import datetime
from functools import wraps
import asyncio
import inspect


@dataclass
class DebugInfo:
    """调试信息"""
    url: str
    title: str
    screenshot_path: Optional[str] = None
    html_path: Optional[str] = None
    console_logs: List[str] = None
    errors: List[str] = None
    timestamp: datetime = None
    
    def __post_init__(self):
        if self.console_logs is None:
            self.console_logs = []
        if self.errors is None:
            self.errors = []
        if self.timestamp is None:
            self.timestamp = datetime.now()


class Debugger:
    """
    调试器
    提供调试辅助功能
    """
    
    def __init__(self, page: Page, debug_dir: str = "./debug"):
        self.page = page
        self.debug_dir = debug_dir
        self._console_messages: List[str] = []
        self._page_errors: List[str] = []
        self._is_monitoring = False
    
    def start_monitoring(self):
        """开始监控控制台和错误"""
        if self._is_monitoring:
            return
        
        self._is_monitoring = True
        
        self.page.on("console", lambda msg: self._console_messages.append(
            f"[{msg.type}] {msg.text}"
        ))
        self.page.on("pageerror", lambda err: self._page_errors.append(str(err)))
        
        logger.info("调试监控已启动")
    
    def stop_monitoring(self):
        """停止监控"""
        self._is_monitoring = False
        self.page.remove_listener("console", lambda msg: None)
        self.page.remove_listener("pageerror", lambda err: None)
    
    def get_console_logs(self) -> List[str]:
        """获取控制台日志"""
        return self._console_messages.copy()
    
    def get_errors(self) -> List[str]:
        """获取页面错误"""
        return self._page_errors.copy()
    
    def clear_logs(self):
        """清除日志"""
        self._console_messages.clear()
        self._page_errors.clear()
    
    async def capture_debug_info(
        self,
        name: str = "debug",
        include_screenshot: bool = True,
        include_html: bool = True,
    ) -> DebugInfo:
        """
        捕获调试信息
        
        Args:
            name: 名称
            include_screenshot: 是否包含截图
            include_html: 是否包含HTML
            
        Returns:
            DebugInfo: 调试信息
        """
        from pathlib import Path
        
        debug_dir = Path(self.debug_dir)
        debug_dir.mkdir(parents=True, exist_ok=True)
        
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        
        screenshot_path = None
        html_path = None
        
        if include_screenshot:
            screenshot_path = str(debug_dir / f"{name}_{timestamp}.png")
            await self.page.screenshot(path=screenshot_path)
        
        if include_html:
            html_path = str(debug_dir / f"{name}_{timestamp}.html")
            html = await self.page.content()
            Path(html_path).write_text(html, encoding='utf-8')
        
        info = DebugInfo(
            url=self.page.url,
            title=await self.page.title(),
            screenshot_path=screenshot_path,
            html_path=html_path,
            console_logs=self._console_messages.copy(),
            errors=self._page_errors.copy(),
        )
        
        logger.info(f"调试信息已捕获: {name}")
        return info
    
    async def highlight_all_clickable(self):
        """高亮所有可点击元素"""
        await self.page.evaluate("""
            () => {
                const clickables = document.querySelectorAll('a, button, input[type="button"], input[type="submit"], [onclick], [role="button"]');
                clickables.forEach(el => {
                    el.style.outline = '2px solid red';
                });
            }
        """)
        logger.debug("已高亮所有可点击元素")
    
    async def highlight_all_inputs(self):
        """高亮所有输入框"""
        await self.page.evaluate("""
            () => {
                const inputs = document.querySelectorAll('input, textarea, select');
                inputs.forEach(el => {
                    el.style.outline = '2px solid blue';
                });
            }
        """)
        logger.debug("已高亮所有输入框")
    
    async def print_element_info(self, selector: str):
        """打印元素信息"""
        element = self.page.locator(selector)
        
        try:
            info = await element.evaluate("""
                el => ({
                    tagName: el.tagName,
                    id: el.id,
                    className: el.className,
                    text: el.textContent?.substring(0, 100),
                    value: el.value,
                    href: el.href,
                    src: el.src,
                    display: getComputedStyle(el).display,
                    visibility: getComputedStyle(el).visibility,
                    position: getBoundingClientRect(el) ? {
                        x: getBoundingClientRect(el).x,
                        y: getBoundingClientRect(el).y,
                        width: getBoundingClientRect(el).width,
                        height: getBoundingClientRect(el).height,
                    } : null
                })
            """)
            
            print(f"\n=== 元素信息: {selector} ===")
            for key, value in info.items():
                print(f"  {key}: {value}")
            
        except Exception as e:
            print(f"无法获取元素信息: {e}")
    
    async def print_page_info(self):
        """打印页面信息"""
        info = await self.page.evaluate("""
            () => ({
                url: location.href,
                title: document.title,
                readyState: document.readyState,
                viewport: {
                    width: window.innerWidth,
                    height: window.innerHeight,
                },
                scroll: {
                    x: window.scrollX,
                    y: window.scrollY,
                },
            })
        """)
        
        print(f"\n=== 页面信息 ===")
        print(f"  URL: {info['url']}")
        print(f"  Title: {info['title']}")
        print(f"  ReadyState: {info['readyState']}")
        print(f"  Viewport: {info['viewport']['width']}x{info['viewport']['height']}")
        print(f"  Scroll: x={info['scroll']['x']}, y={info['scroll']['y']}")


def retry_on_error(
    max_retries: int = 3,
    delay: float = 1.0,
    exceptions: tuple = (Exception,),
    on_retry: Optional[Callable] = None,
):
    """
    错误重试装饰器
    
    Args:
        max_retries: 最大重试次数
        delay: 重试延迟
        exceptions: 要捕获的异常类型
        on_retry: 重试时的回调
        
    Example:
        @retry_on_error(max_retries=3, delay=1.0)
        async def unstable_operation():
            ...
    """
    def decorator(func):
        @wraps(func)
        async def async_wrapper(*args, **kwargs):
            last_error = None
            
            for attempt in range(max_retries):
                try:
                    return await func(*args, **kwargs)
                except exceptions as e:
                    last_error = e
                    logger.warning(
                        f"操作失败: {func.__name__}, "
                        f"尝试 {attempt + 1}/{max_retries}, error={e}"
                    )
                    
                    if on_retry:
                        on_retry(attempt, e)
                    
                    if attempt < max_retries - 1:
                        await asyncio.sleep(delay)
            
            raise last_error
        
        @wraps(func)
        def sync_wrapper(*args, **kwargs):
            import time
            last_error = None
            
            for attempt in range(max_retries):
                try:
                    return func(*args, **kwargs)
                except exceptions as e:
                    last_error = e
                    logger.warning(
                        f"操作失败: {func.__name__}, "
                        f"尝试 {attempt + 1}/{max_retries}, error={e}"
                    )
                    
                    if on_retry:
                        on_retry(attempt, e)
                    
                    if attempt < max_retries - 1:
                        time.sleep(delay)
            
            raise last_error
        
        if asyncio.iscoroutinefunction(func):
            return async_wrapper
        return sync_wrapper
    
    return decorator


class ErrorHandler:
    """
    错误处理器
    统一的错误处理和日志记录
    """
    
    def __init__(self, page: Page, screenshot_on_error: bool = True):
        self.page = page
        self.screenshot_on_error = screenshot_on_error
        self._errors: List[Dict[str, Any]] = []
    
    async def handle_error(
        self,
        error: Exception,
        context: Optional[str] = None,
        screenshot_name: Optional[str] = None,
    ) -> Dict[str, Any]:
        """
        处理错误
        
        Args:
            error: 异常对象
            context: 上下文信息
            screenshot_name: 截图名称
            
        Returns:
            Dict: 错误信息
        """
        timestamp = datetime.now()
        
        error_info = {
            "timestamp": timestamp.isoformat(),
            "error_type": type(error).__name__,
            "error_message": str(error),
            "context": context,
            "url": self.page.url,
            "traceback": traceback.format_exc(),
        }
        
        # 截图
        if self.screenshot_on_error:
            screenshot_path = f"./debug/error_{timestamp.strftime('%Y%m%d_%H%M%S')}.png"
            try:
                await self.page.screenshot(path=screenshot_path)
                error_info["screenshot"] = screenshot_path
            except:
                pass
        
        self._errors.append(error_info)
        
        logger.error(
            f"错误处理: {error_info['error_type']}: {error_info['error_message']}"
        )
        
        return error_info
    
    def get_errors(self) -> List[Dict[str, Any]]:
        """获取所有错误"""
        return self._errors.copy()
    
    def clear_errors(self):
        """清除错误记录"""
        self._errors.clear()
    
    def has_errors(self) -> bool:
        """是否有错误"""
        return len(self._errors) > 0
    
    def get_error_summary(self) -> Dict[str, int]:
        """获取错误统计"""
        summary = {}
        for error in self._errors:
            error_type = error["error_type"]
            summary[error_type] = summary.get(error_type, 0) + 1
        return summary


class AssertionHelper:
    """
    断言辅助工具
    提供更友好的断言方法
    """
    
    def __init__(self, page: Page):
        self.page = page
    
    async def element_exists(self, selector: str, message: str = "") -> bool:
        """断言元素存在"""
        count = await self.page.locator(selector).count()
        if count == 0:
            raise AssertionError(
                f"元素不存在: {selector}. {message}"
            )
        return True
    
    async def element_not_exists(self, selector: str, message: str = "") -> bool:
        """断言元素不存在"""
        count = await self.page.locator(selector).count()
        if count > 0:
            raise AssertionError(
                f"元素存在（预期不存在）: {selector}. {message}"
            )
        return True
    
    async def element_visible(self, selector: str, message: str = "") -> bool:
        """断言元素可见"""
        locator = self.page.locator(selector)
        if not await locator.is_visible():
            raise AssertionError(
                f"元素不可见: {selector}. {message}"
            )
        return True
    
    async def element_hidden(self, selector: str, message: str = "") -> bool:
        """断言元素隐藏"""
        locator = self.page.locator(selector)
        if await locator.is_visible():
            raise AssertionError(
                f"元素可见（预期隐藏）: {selector}. {message}"
            )
        return True
    
    async def element_enabled(self, selector: str, message: str = "") -> bool:
        """断言元素可用"""
        locator = self.page.locator(selector)
        if not await locator.is_enabled():
            raise AssertionError(
                f"元素禁用: {selector}. {message}"
            )
        return True
    
    async def element_disabled(self, selector: str, message: str = "") -> bool:
        """断言元素禁用"""
        locator = self.page.locator(selector)
        if await locator.is_enabled():
            raise AssertionError(
                f"元素可用（预期禁用）: {selector}. {message}"
            )
        return True
    
    async def text_equals(
        self, selector: str, expected: str, message: str = ""
    ) -> bool:
        """断言文本相等"""
        actual = await self.page.locator(selector).text_content()
        actual = (actual or "").strip()
        
        if actual != expected:
            raise AssertionError(
                f"文本不匹配: 期望 '{expected}', 实际 '{actual}'. {message}"
            )
        return True
    
    async def text_contains(
        self, selector: str, expected: str, message: str = ""
    ) -> bool:
        """断言文本包含"""
        actual = await self.page.locator(selector).text_content()
        actual = actual or ""
        
        if expected not in actual:
            raise AssertionError(
                f"文本不包含: 期望包含 '{expected}', 实际 '{actual[:100]}'. {message}"
            )
        return True
    
    async def value_equals(
        self, selector: str, expected: str, message: str = ""
    ) -> bool:
        """断言值相等"""
        actual = await self.page.locator(selector).input_value()
        
        if actual != expected:
            raise AssertionError(
                f"值不匹配: 期望 '{expected}', 实际 '{actual}'. {message}"
            )
        return True
    
    async def url_equals(self, expected: str, message: str = "") -> bool:
        """断言URL相等"""
        actual = self.page.url
        
        if actual != expected:
            raise AssertionError(
                f"URL不匹配: 期望 '{expected}', 实际 '{actual}'. {message}"
            )
        return True
    
    async def url_contains(self, expected: str, message: str = "") -> bool:
        """断言URL包含"""
        actual = self.page.url
        
        if expected not in actual:
            raise AssertionError(
                f"URL不包含: 期望包含 '{expected}', 实际 '{actual}'. {message}"
            )
        return True
    
    async def title_equals(self, expected: str, message: str = "") -> bool:
        """断言标题相等"""
        actual = await self.page.title()
        
        if actual != expected:
            raise AssertionError(
                f"标题不匹配: 期望 '{expected}', 实际 '{actual}'. {message}"
            )
        return True
    
    async def title_contains(self, expected: str, message: str = "") -> bool:
        """断言标题包含"""
        actual = await self.page.title()
        
        if expected not in actual:
            raise AssertionError(
                f"标题不包含: 期望包含 '{expected}', 实际 '{actual}'. {message}"
            )
        return True
    
    async def count_equals(
        self, selector: str, expected: int, message: str = ""
    ) -> bool:
        """断言元素数量相等"""
        actual = await self.page.locator(selector).count()
        
        if actual != expected:
            raise AssertionError(
                f"元素数量不匹配: 期望 {expected}, 实际 {actual}. {message}"
            )
        return True



"""
文件操作模块
支持文件上传、下载、拖拽上传
"""
import asyncio
from typing import Optional, List, Callable
from pathlib import Path
from playwright.async_api import Page, Locator, Download, FileChooser
from loguru import logger
from dataclasses import dataclass
from datetime import datetime


@dataclass
class DownloadResult:
    """下载结果"""
    success: bool
    filename: str
    save_path: str
    size_bytes: int
    duration_ms: float
    error: Optional[str] = None


class FileHandler:
    """文件处理器"""
    
    def __init__(self, page: Page):
        self.page = page
    
    async def upload(self, selector: str, files: List[str], timeout: float = 30) -> bool:
        """上传文件"""
        logger.info(f"上传文件: selector={selector}, files={files}")
        
        locator = self.page.locator(selector)
        
        for f in files:
            if not Path(f).exists():
                logger.error(f"文件不存在: {f}")
                return False
        
        try:
            await locator.set_input_files(files, timeout=timeout * 1000)
            logger.info(f"文件上传成功: {len(files)} 个文件")
            return True
        except Exception as e:
            logger.error(f"文件上传失败: {e}")
            return False
    
    async def download(
        self,
        trigger_selector: str,
        save_dir: str = "./downloads",
        timeout: float = 60,
    ) -> DownloadResult:
        """下载文件"""
        logger.info(f"下载文件: trigger={trigger_selector}")
        
        Path(save_dir).mkdir(parents=True, exist_ok=True)
        start_time = datetime.now()
        
        try:
            async with self.page.expect_download(timeout=timeout * 1000) as download_info:
                await self.page.click(trigger_selector)
            
            download = await download_info.value
            save_path = str(Path(save_dir) / download.suggested_filename)
            await download.save_as(save_path)
            
            file_size = Path(save_path).stat().st_size
            duration = (datetime.now() - start_time).total_seconds() * 1000
            
            logger.info(f"文件下载成功: {save_path}")
            
            return DownloadResult(
                success=True,
                filename=download.suggested_filename,
                save_path=save_path,
                size_bytes=file_size,
                duration_ms=duration,
            )
            
        except Exception as e:
            duration = (datetime.now() - start_time).total_seconds() * 1000
            logger.error(f"文件下载失败: {e}")
            return DownloadResult(
                success=False,
                filename="",
                save_path="",
                size_bytes=0,
                duration_ms=duration,
                error=str(e),
            )
    
    async def screenshot_element(self, selector: str, save_path: str) -> bool:
        """元素截图"""
        try:
            locator = self.page.locator(selector)
            await locator.screenshot(path=save_path)
            logger.info(f"元素截图成功: {save_path}")
            return True
        except Exception as e:
            logger.error(f"元素截图失败: {e}")
            return False
    
    async def pdf(self, save_path: str, format: str = "A4") -> bool:
        """页面保存为PDF"""
        try:
            await self.page.pdf(
                path=save_path,
                format=format,
                print_background=True,
                margin={"top": "1cm", "bottom": "1cm", "left": "1cm", "right": "1cm"},
            )
            logger.info(f"PDF保存成功: {save_path}")
            return True
        except Exception as e:
            logger.error(f"PDF保存失败: {e}")
            return False


class BatchUploader:
    """批量上传器"""
    
    def __init__(self, page: Page):
        self.page = page
        self.file_handler = FileHandler(page)
    
    async def upload_batch(
        self,
        selector: str,
        files: List[str],
        batch_size: int = 5,
        delay: float = 1.0,
    ) -> dict:
        """批量上传文件"""
        results = {"total": len(files), "success": 0, "failed": 0, "errors": []}
        
        for i in range(0, len(files), batch_size):
            batch = files[i:i + batch_size]
            
            for file in batch:
                try:
                    success = await self.file_handler.upload(selector, [file])
                    if success:
                        results["success"] += 1
                    else:
                        results["failed"] += 1
                except Exception as e:
                    results["failed"] += 1
                    results["errors"].append({"file": file, "error": str(e)})
            
            if i + batch_size < len(files):
                await asyncio.sleep(delay)
        
        logger.info(f"批量上传完成: 成功={results['success']}, 失败={results['failed']}")
        return results



"""
自动化辅助类
整合 automation 子模块，提供统一访问入口
"""
from typing import Callable, Optional
from playwright.async_api import Page

from ..automation import (
    InfiniteScrollHandler,
    InfiniteScrollConfig,
    PaginationNavigator,
    PaginationConfig,
    IframeManager,
    DragDropHelper,
    DragConfig,
    FileManager,
    FileResult,
    UploadConfig,
    DownloadConfig,
)


class AutomationHelper:
    """
    自动化辅助类
    整合 automation 子模块

    使用方式:
        browser.automation.infinite_scroll  # InfiniteScrollHandler
        browser.automation.pagination       # PaginationNavigator
        browser.automation.iframe           # IframeManager
        browser.automation.drag_drop        # DragDropHelper
        browser.automation.file             # FileManager
    """

    def __init__(self, get_page: Callable[[], Page]):
        """
        初始化自动化辅助类

        Args:
            get_page: 获取当前 Page 对象的回调函数
        """
        self._get_page = get_page
        self._infinite_scroll: Optional[InfiniteScrollHandler] = None
        self._pagination: Optional[PaginationNavigator] = None
        self._iframe: Optional[IframeManager] = None
        self._drag_drop: Optional[DragDropHelper] = None
        self._file: Optional[FileManager] = None

    @property
    def infinite_scroll(self) -> InfiniteScrollHandler:
        """
        获取无限滚动处理器

        Returns:
            InfiniteScrollHandler: 无限滚动处理器实例
        """
        if self._infinite_scroll is None:
            self._infinite_scroll = InfiniteScrollHandler(self._get_page())
        return self._infinite_scroll

    @property
    def pagination(self) -> PaginationNavigator:
        """
        获取分页导航器

        Returns:
            PaginationNavigator: 分页导航器实例
        """
        if self._pagination is None:
            self._pagination = PaginationNavigator(self._get_page())
        return self._pagination

    @property
    def iframe(self) -> IframeManager:
        """
        获取 iframe 管理器

        Returns:
            IframeManager: iframe 管理器实例
        """
        if self._iframe is None:
            self._iframe = IframeManager(self._get_page())
        return self._iframe

    @property
    def drag_drop(self) -> DragDropHelper:
        """
        获取拖拽操作助手

        Returns:
            DragDropHelper: 拖拽操作助手实例
        """
        if self._drag_drop is None:
            self._drag_drop = DragDropHelper(self._get_page())
        return self._drag_drop

    @property
    def file(self) -> FileManager:
        """
        获取文件管理器

        Returns:
            FileManager: 文件管理器实例
        """
        if self._file is None:
            self._file = FileManager(self._get_page())
        return self._file

    def reset(self):
        """
        重置所有实例
        当切换标签页时调用，确保使用正确的 Page 对象
        """
        self._infinite_scroll = None
        self._pagination = None
        self._iframe = None
        self._drag_drop = None
        self._file = None



"""
iframe 管理器
iframe 深度穿透，自动切换上下文
"""
from dataclasses import dataclass
from typing import Optional, List, Callable, TypeVar, Any
from playwright.async_api import Page, Frame, Locator
from loguru import logger

T = TypeVar('T')


@dataclass
class IframeInfo:
    """iframe 信息"""
    name: str
    selector: str
    index: int
    depth: int
    src: Optional[str] = None


class IframeManager:
    """
    iframe 管理器
    处理 iframe 的查找、切换、操作
    """

    def __init__(self, page: Page):
        self.page = page
        self._frame_stack: List[Frame] = []
        self._current_frame: Optional[Frame] = None

    # ==================== iframe 查找 ====================

    async def get_iframe_info_list(self) -> List[IframeInfo]:
        """获取所有 iframe 信息"""
        frames = self.page.frames
        info_list: List[IframeInfo] = []

        for i, frame in enumerate(frames):
            if frame == self.page.main_frame:
                continue

            info = await self._get_frame_info(frame, i)
            info_list.append(info)

        return info_list

    async def _get_frame_info(self, frame: Frame, index: int) -> IframeInfo:
        """获取单个 iframe 信息"""
        name = frame.name or f"iframe-{index}"
        selector = await self._find_frame_selector(frame)
        depth = self._calculate_frame_depth(frame)
        src = await self._get_frame_src(frame)

        return IframeInfo(
            name=name,
            selector=selector,
            index=index,
            depth=depth,
            src=src,
        )

    async def _find_frame_selector(self, frame: Frame) -> str:
        """查找 iframe 选择器"""
        name = frame.name
        if name:
            return f'iframe[name="{name}"]'

        src = await self._get_frame_src(frame)
        if src:
            return f'iframe[src*="{src[:50]}"]'

        return "iframe"

    async def _get_frame_src(self, frame: Frame) -> Optional[str]:
        """获取 iframe 的 src"""
        try:
            frame_element = await frame.frame_element()
            src = await frame_element.get_attribute("src")
            return src
        except Exception:
            return None

    def _calculate_frame_depth(self, frame: Frame) -> int:
        """计算 iframe 嵌套深度"""
        depth = 0
        current = frame.parent_frame

        while current and current != self.page.main_frame:
            depth += 1
            current = current.parent_frame

        return depth

    # ==================== iframe 切换 ====================

    async def enter_iframe(self, selector: str) -> Optional[Frame]:
        """进入 iframe"""
        try:
            frame_element = await self.page.locator(selector).element_handle()
            if not frame_element:
                return None

            content_frame = await frame_element.content_frame()
            if not content_frame:
                return None

            self._frame_stack.append(self._current_frame or self.page.main_frame)
            self._current_frame = content_frame

            logger.debug(f"进入 iframe: {selector}")
            return content_frame
        except Exception as e:
            logger.debug(f"进入 iframe 失败: {selector}, error={e}")
            return None

    async def enter_iframe_by_name(self, name: str) -> Optional[Frame]:
        """通过名称进入 iframe"""
        return await self.enter_iframe(f'iframe[name="{name}"]')

    async def enter_iframe_by_index(self, index: int) -> Optional[Frame]:
        """通过索引进入 iframe"""
        return await self.enter_iframe(f"iframe >> nth={index}")

    def exit_iframe(self) -> Optional[Frame]:
        """退出当前 iframe"""
        if not self._frame_stack:
            self._current_frame = None
            return self.page.main_frame

        self._current_frame = self._frame_stack.pop()
        logger.debug(f"退出 iframe, 当前层级: {len(self._frame_stack)}")
        return self._current_frame

    def exit_to_main_frame(self):
        """退出到主框架"""
        self._frame_stack.clear()
        self._current_frame = None
        logger.debug("退出到主框架")

    def get_current_frame(self) -> Frame:
        """获取当前框架"""
        return self._current_frame or self.page.main_frame

    def get_stack_depth(self) -> int:
        """获取框架堆栈深度"""
        return len(self._frame_stack)

    # ==================== iframe 操作 ====================

    def locate(self, selector: str) -> Locator:
        """在当前框架中定位元素"""
        frame = self.get_current_frame()
        return frame.locator(selector)

    async def execute_in_frame(
        self,
        selector: str,
        action: Callable[[Frame], Any]
    ) -> Any:
        """在指定 iframe 中执行操作"""
        frame = await self.enter_iframe(selector)
        if not frame:
            return None

        try:
            if asyncio.iscoroutinefunction(action):
                return await action(frame)
            else:
                return action(frame)
        finally:
            self.exit_iframe()

    async def execute_in_nested_frame(
        self,
        selectors: List[str],
        action: Callable[[Frame], Any]
    ) -> Any:
        """在嵌套 iframe 中执行操作"""
        import asyncio

        entered_frames: List[str] = []

        try:
            # 依次进入嵌套的 iframe
            for selector in selectors:
                frame = await self.enter_iframe(selector)
                if not frame:
                    raise RuntimeError(f"无法进入 iframe: {selector}")
                entered_frames.append(selector)

            # 执行操作
            current_frame = self.get_current_frame()
            if asyncio.iscoroutinefunction(action):
                return await action(current_frame)
            else:
                return action(current_frame)
        except Exception as e:
            logger.error(f"嵌套 iframe 操作失败: {e}")
            return None
        finally:
            # 退出所有进入的 iframe
            for _ in entered_frames:
                self.exit_iframe()

    # ==================== iframe 等待 ====================

    async def wait_for_iframe(
        self,
        selector: str,
        timeout: float = 30000
    ) -> Optional[Frame]:
        """等待 iframe 加载"""
        try:
            frame_element = self.page.locator(selector)
            await frame_element.wait_for(state="attached", timeout=timeout)

            handle = await frame_element.element_handle()
            if not handle:
                return None

            frame = await handle.content_frame()
            if not frame:
                return None

            # 等待 iframe 内容加载
            try:
                await frame.wait_for_load_state("domcontentloaded", timeout=timeout)
            except Exception:
                pass

            return frame
        except Exception:
            return None

    async def wait_for_element_in_iframe(
        self,
        iframe_selector: str,
        element_selector: str,
        timeout: float = 30000
    ) -> Optional[Locator]:
        """等待 iframe 内元素出现"""
        try:
            frame = await self.wait_for_iframe(iframe_selector, timeout)
            if not frame:
                return None

            element = frame.locator(element_selector)
            await element.wait_for(state="visible", timeout=timeout)

            return element
        except Exception:
            return None

    # ==================== iframe 状态检测 ====================

    async def has_iframe(self, selector: str) -> bool:
        """检测 iframe 是否存在"""
        count = await self.page.locator(selector).count()
        return count > 0

    async def is_iframe_visible(self, selector: str) -> bool:
        """检测 iframe 是否可见"""
        try:
            return await self.page.locator(selector).is_visible()
        except Exception:
            return False

    async def get_iframe_count(self) -> int:
        """获取 iframe 数量"""
        return await self.page.locator("iframe").count()

    async def is_iframe_interactable(self, selector: str) -> bool:
        """检测 iframe 是否可交互"""
        try:
            frame = await self.wait_for_iframe(selector, 5000)
            if not frame:
                return False

            # 检查 iframe 是否有内容
            body = frame.locator("body")
            content = await body.inner_html()
            return len(content) > 0
        except Exception:
            return False

    # ==================== iframe 内容操作 ====================

    async def get_iframe_content(self, selector: str) -> Optional[str]:
        """获取 iframe 内容"""
        try:
            frame = await self.wait_for_iframe(selector)
            if not frame:
                return None

            return await frame.content()
        except Exception:
            return None

    async def click_in_iframe(
        self,
        iframe_selector: str,
        element_selector: str
    ) -> bool:
        """在 iframe 中点击元素"""
        try:
            element = await self.wait_for_element_in_iframe(
                iframe_selector, element_selector
            )
            if not element:
                return False

            await element.click()
            logger.debug(f"在 iframe 中点击: {element_selector}")
            return True
        except Exception:
            return False

    async def fill_in_iframe(
        self,
        iframe_selector: str,
        element_selector: str,
        value: str
    ) -> bool:
        """在 iframe 中填写表单"""
        try:
            element = await self.wait_for_element_in_iframe(
                iframe_selector, element_selector
            )
            if not element:
                return False

            await element.fill(value)
            logger.debug(f"在 iframe 中填写: {element_selector}")
            return True
        except Exception:
            return False


import asyncio


def create_iframe_manager(page: Page) -> IframeManager:
    """创建 iframe 管理器实例"""
    return IframeManager(page)



"""
日志和追踪模块
提供详细的日志记录和追踪功能
"""
import json
from typing import Optional, List, Dict, Any
from pathlib import Path
from datetime import datetime
from loguru import logger
from dataclasses import dataclass, field


@dataclass
class TraceEvent:
    """追踪事件"""
    name: str
    category: str
    timestamp: datetime
    duration_ms: Optional[float] = None
    data: Dict[str, Any] = field(default_factory=dict)


class TraceCollector:
    """
    追踪收集器
    收集和分析操作追踪
    """
    
    def __init__(self, output_dir: str = "./traces"):
        self.output_dir = Path(output_dir)
        self.output_dir.mkdir(parents=True, exist_ok=True)
        self._events: List[TraceEvent] = []
        self._current_trace: Optional[str] = None
    
    def start_trace(self, name: str):
        """开始追踪"""
        self._current_trace = name
        self._events.clear()
        
        event = TraceEvent(
            name=f"trace_start",
            category="trace",
            timestamp=datetime.now(),
            data={"trace_name": name},
        )
        self._events.append(event)
        
        logger.info(f"开始追踪: {name}")
    
    def end_trace(self) -> List[TraceEvent]:
        """结束追踪"""
        event = TraceEvent(
            name="trace_end",
            category="trace",
            timestamp=datetime.now(),
        )
        self._events.append(event)
        
        logger.info(f"结束追踪: {self._current_trace}")
        
        events = self._events.copy()
        self._current_trace = None
        
        return events
    
    def record_event(
        self,
        name: str,
        category: str = "action",
        duration_ms: float = None,
        **kwargs,
    ):
        """记录事件"""
        event = TraceEvent(
            name=name,
            category=category,
            timestamp=datetime.now(),
            duration_ms=duration_ms,
            data=kwargs,
        )
        self._events.append(event)
    
    def record_action(self, action: str, selector: str = None, **kwargs):
        """记录操作"""
        self.record_event(
            name=action,
            category="action",
            selector=selector,
            **kwargs,
        )
    
    def record_navigation(self, url: str, duration_ms: float = None):
        """记录导航"""
        self.record_event(
            name="navigate",
            category="navigation",
            duration_ms=duration_ms,
            url=url,
        )
    
    def record_wait(self, condition: str, duration_ms: float):
        """记录等待"""
        self.record_event(
            name="wait",
            category="wait",
            duration_ms=duration_ms,
            condition=condition,
        )
    
    def get_events(self, category: str = None) -> List[TraceEvent]:
        """获取事件"""
        if category:
            return [e for e in self._events if e.category == category]
        return self._events.copy()
    
    def get_summary(self) -> Dict[str, Any]:
        """获取摘要"""
        categories = {}
        total_duration = 0
        
        for event in self._events:
            if event.category not in categories:
                categories[event.category] = {"count": 0, "total_duration": 0}
            
            categories[event.category]["count"] += 1
            
            if event.duration_ms:
                categories[event.category]["total_duration"] += event.duration_ms
                total_duration += event.duration_ms
        
        return {
            "total_events": len(self._events),
            "total_duration_ms": total_duration,
            "categories": categories,
        }
    
    def export_chrome_trace(self, filepath: str = None):
        """导出Chrome Tracing格式"""
        events = []
        
        for event in self._events:
            ts = int(event.timestamp.timestamp() * 1000000)  # 微秒
            
            trace_event = {
                "name": event.name,
                "cat": event.category,
                "ph": "i" if event.duration_ms is None else "X",
                "ts": ts,
                "pid": 1,
                "tid": 1,
                "args": event.data,
            }
            
            if event.duration_ms:
                trace_event["dur"] = int(event.duration_ms * 1000)  # 微秒
            
            events.append(trace_event)
        
        filepath = filepath or str(self.output_dir / f"trace_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json")
        
        with open(filepath, 'w', encoding='utf-8') as f:
            json.dump(events, f, ensure_ascii=False)
        
        logger.info(f"追踪已导出: {filepath}")
        return filepath


class ActionLogger:
    """
    操作日志记录器
    详细记录所有浏览器操作
    """
    
    def __init__(self, log_file: str = None):
        self.log_file = Path(log_file) if log_file else None
        self._logs: List[Dict[str, Any]] = []
    
    def log(
        self,
        action: str,
        selector: str = None,
        value: Any = None,
        success: bool = True,
        error: str = None,
    ):
        """记录操作"""
        entry = {
            "timestamp": datetime.now().isoformat(),
            "action": action,
            "selector": selector,
            "value": str(value)[:100] if value else None,
            "success": success,
            "error": error,
        }
        
        self._logs.append(entry)
        
        if self.log_file:
            with open(self.log_file, 'a', encoding='utf-8') as f:
                f.write(json.dumps(entry, ensure_ascii=False) + '\n')
    
    def get_logs(self, action: str = None) -> List[Dict]:
        """获取日志"""
        if action:
            return [l for l in self._logs if l["action"] == action]
        return self._logs.copy()
    
    def get_failed_actions(self) -> List[Dict]:
        """获取失败的操作"""
        return [l for l in self._logs if not l["success"]]
    
    def clear(self):
        """清除日志"""
        self._logs.clear()


class PerformanceLogger:
    """
    性能日志记录器
    记录性能相关数据
    """
    
    def __init__(self):
        self._metrics: Dict[str, List[float]] = {}
    
    def record(self, metric_name: str, value: float):
        """记录指标"""
        if metric_name not in self._metrics:
            self._metrics[metric_name] = []
        self._metrics[metric_name].append(value)
    
    def get_stats(self, metric_name: str) -> Dict[str, float]:
        """获取统计"""
        if metric_name not in self._metrics:
            return {}
        
        values = self._metrics[metric_name]
        
        return {
            "count": len(values),
            "avg": sum(values) / len(values),
            "min": min(values),
            "max": max(values),
            "sum": sum(values),
        }
    
    def get_all_stats(self) -> Dict[str, Dict[str, float]]:
        """获取所有统计"""
        return {name: self.get_stats(name) for name in self._metrics}


class ErrorTracker:
    """
    错误追踪器
    追踪和分析错误
    """
    
    def __init__(self):
        self._errors: List[Dict[str, Any]] = []
    
    def track(
        self,
        error: Exception,
        context: Dict[str, Any] = None,
        screenshot: str = None,
    ):
        """追踪错误"""
        import traceback
        
        entry = {
            "timestamp": datetime.now().isoformat(),
            "error_type": type(error).__name__,
            "error_message": str(error),
            "traceback": traceback.format_exc(),
            "context": context or {},
            "screenshot": screenshot,
        }
        
        self._errors.append(entry)
        logger.error(f"错误追踪: {entry['error_type']}: {entry['error_message']}")
    
    def get_errors(self, error_type: str = None) -> List[Dict]:
        """获取错误"""
        if error_type:
            return [e for e in self._errors if e["error_type"] == error_type]
        return self._errors.copy()
    
    def get_error_summary(self) -> Dict[str, int]:
        """获取错误摘要"""
        summary = {}
        for error in self._errors:
            error_type = error["error_type"]
            summary[error_type] = summary.get(error_type, 0) + 1
        return summary
    
    def has_errors(self) -> bool:
        """是否有错误"""
        return len(self._errors) > 0
    
    def clear(self):
        """清除错误"""
        self._errors.clear()


class StepReporter:
    """
    步骤报告器
    生成测试步骤报告
    """
    
    def __init__(self):
        self._steps: List[Dict[str, Any]] = []
        self._current_step: Optional[int] = None
    
    def start_step(self, name: str, description: str = None):
        """开始步骤"""
        step = {
            "name": name,
            "description": description,
            "start_time": datetime.now(),
            "end_time": None,
            "status": "running",
            "error": None,
        }
        
        self._steps.append(step)
        self._current_step = len(self._steps) - 1
        
        logger.info(f"开始步骤: {name}")
    
    def end_step(self, success: bool = True, error: str = None):
        """结束步骤"""
        if self._current_step is not None:
            step = self._steps[self._current_step]
            step["end_time"] = datetime.now()
            step["status"] = "passed" if success else "failed"
            step["error"] = error
            
            duration = (step["end_time"] - step["start_time"]).total_seconds()
            
            status_emoji = "✅" if success else "❌"
            logger.info(f"{status_emoji} 结束步骤: {step['name']} ({duration:.2f}s)")
            
            self._current_step = None
    
    def get_report(self) -> Dict[str, Any]:
        """获取报告"""
        total = len(self._steps)
        passed = sum(1 for s in self._steps if s["status"] == "passed")
        failed = total - passed
        
        steps_data = []
        for step in self._steps:
            duration = 0
            if step["end_time"] and step["start_time"]:
                duration = (step["end_time"] - step["start_time"]).total_seconds()
            
            steps_data.append({
                "name": step["name"],
                "description": step["description"],
                "status": step["status"],
                "duration_seconds": duration,
                "error": step["error"],
            })
        
        return {
            "total_steps": total,
            "passed": passed,
            "failed": failed,
            "steps": steps_data,
        }
    
    def export_html(self, filepath: str):
        """导出HTML报告"""
        report = self.get_report()
        
        html = f"""
<!DOCTYPE html>
<html>
<head>
    <title>Step Report</title>
    <style>
        body {{ font-family: Arial, sans-serif; padding: 20px; }}
        .summary {{ background: #f5f5f5; padding: 20px; margin-bottom: 20px; }}
        .step {{ padding: 15px; margin-bottom: 10px; border-radius: 5px; }}
        .step.passed {{ background: #e8f5e9; border-left: 4px solid #4CAF50; }}
        .step.failed {{ background: #ffebee; border-left: 4px solid #f44336; }}
        .step-name {{ font-weight: bold; margin-bottom: 5px; }}
        .step-duration {{ color: #666; font-size: 14px; }}
    </style>
</head>
<body>
    <h1>Step Report</h1>
    <div class="summary">
        <p>Total: {report['total_steps']}</p>
        <p>Passed: {report['passed']}</p>
        <p>Failed: {report['failed']}</p>
    </div>
    <div class="steps">
"""
        
        for step in report["steps"]:
            html += f"""
        <div class="step {step['status']}">
            <div class="step-name">{step['name']}</div>
            <div class="step-duration">{step['duration_seconds']:.2f}s</div>
            {f'<div class="error">{step["error"]}</div>' if step['error'] else ''}
        </div>
"""
        
        html += """
    </div>
</body>
</html>
"""
        
        Path(filepath).parent.mkdir(parents=True, exist_ok=True)
        Path(filepath).write_text(html, encoding='utf-8')
        
        logger.info(f"步骤报告已导出: {filepath}")


"""
操作录制与回放模块
记录用户操作，生成代码
"""
import json
import asyncio
from typing import Optional, List, Dict, Any, Callable
from playwright.async_api import Page
from loguru import logger
from dataclasses import dataclass, field
from datetime import datetime
from pathlib import Path
from enum import Enum


class ActionType(Enum):
    """操作类型"""
    CLICK = "click"
    FILL = "fill"
    TYPE = "type"
    SELECT = "select"
    CHECK = "check"
    UNCHECK = "unchecked"
    NAVIGATE = "navigate"
    WAIT = "wait"
    HOVER = "hover"
    SCROLL = "scroll"
    KEYBOARD = "keyboard"
    SCREENSHOT = "screenshot"


@dataclass
class RecordedAction:
    """录制的操作"""
    action_type: ActionType
    timestamp: float
    selector: Optional[str] = None
    value: Optional[str] = None
    x: Optional[int] = None
    y: Optional[int] = None
    key: Optional[str] = None
    options: Dict[str, Any] = field(default_factory=dict)
    
    def to_dict(self) -> Dict[str, Any]:
        return {
            "action_type": self.action_type.value,
            "timestamp": self.timestamp,
            "selector": self.selector,
            "value": self.value,
            "x": self.x,
            "y": self.y,
            "key": self.key,
            "options": self.options,
        }
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "RecordedAction":
        return cls(
            action_type=ActionType(data["action_type"]),
            timestamp=data["timestamp"],
            selector=data.get("selector"),
            value=data.get("value"),
            x=data.get("x"),
            y=data.get("y"),
            key=data.get("key"),
            options=data.get("options", {}),
        )


class ActionRecorder:
    """
    操作录制器
    记录用户在页面上的操作
    """
    
    def __init__(self, page: Page):
        self.page = page
        self.actions: List[RecordedAction] = []
        self._is_recording = False
        self._start_time: Optional[float] = None
        self._listeners: List[Callable] = []
    
    async def start(self):
        """开始录制"""
        if self._is_recording:
            logger.warning("录制已在进行中")
            return
        
        self._is_recording = True
        self._start_time = datetime.now().timestamp()
        self.actions.clear()
        
        # 注入录制脚本
        await self.page.evaluate("""
            () => {
                window.__recordedActions = window.__recordedActions || [];
                window.__recording = true;
                
                // 点击事件
                document.addEventListener('click', (e) => {
                    if (!window.__recording) return;
                    
                    const element = e.target;
                    const selector = getSelector(element);
                    
                    window.__recordedActions.push({
                        action_type: 'click',
                        selector: selector,
                        timestamp: Date.now()
                    });
                }, true);
                
                // 输入事件
                document.addEventListener('input', (e) => {
                    if (!window.__recording) return;
                    
                    const element = e.target;
                    const selector = getSelector(element);
                    
                    window.__recordedActions.push({
                        action_type: 'fill',
                        selector: selector,
                        value: element.value,
                        timestamp: Date.now()
                    });
                }, true);
                
                // 选择事件
                document.addEventListener('change', (e) => {
                    if (!window.__recording) return;
                    
                    const element = e.target;
                    if (element.tagName === 'SELECT') {
                        const selector = getSelector(element);
                        window.__recordedActions.push({
                            action_type: 'select',
                            selector: selector,
                            value: element.value,
                            timestamp: Date.now()
                        });
                    }
                }, true);
                
                // 生成选择器
                function getSelector(element) {
                    if (element.id) return '#' + element.id;
                    if (element.name) return '[name="' + element.name + '"]';
                    if (element.className) {
                        const classes = element.className.split(' ').filter(c => c);
                        if (classes.length > 0) return '.' + classes[0];
                    }
                    
                    // 使用路径选择器
                    const path = [];
                    let current = element;
                    
                    while (current && current !== document.body) {
                        let selector = current.tagName.toLowerCase();
                        if (current.id) {
                            selector = '#' + current.id;
                            path.unshift(selector);
                            break;
                        }
                        
                        const siblings = current.parentElement.children;
                        let index = 1;
                        for (const sibling of siblings) {
                            if (sibling === current) break;
                            if (sibling.tagName === current.tagName) index++;
                        }
                        
                        selector += ':nth-of-type(' + index + ')';
                        path.unshift(selector);
                        current = current.parentElement;
                    }
                    
                    return path.join(' > ');
                }
                
                window.getSelector = getSelector;
            }
        """)
        
        logger.info("开始录制操作")
    
    async def stop(self) -> List[RecordedAction]:
        """停止录制"""
        if not self._is_recording:
            logger.warning("没有正在进行的录制")
            return []
        
        self._is_recording = False
        
        # 获取录制的操作
        recorded = await self.page.evaluate("""
            () => {
                window.__recording = false;
                return window.__recordedActions || [];
            }
        """)
        
        # 转换为RecordedAction
        for action_data in recorded:
            action = RecordedAction(
                action_type=ActionType(action_data["action_type"]),
                timestamp=action_data["timestamp"],
                selector=action_data.get("selector"),
                value=action_data.get("value"),
            )
            self.actions.append(action)
        
        logger.info(f"录制完成: {len(self.actions)} 个操作")
        return self.actions
    
    async def record_action(
        self,
        action_type: ActionType,
        selector: Optional[str] = None,
        value: Optional[str] = None,
    ):
        """手动记录操作"""
        action = RecordedAction(
            action_type=action_type,
            timestamp=datetime.now().timestamp(),
            selector=selector,
            value=value,
        )
        self.actions.append(action)
        logger.debug(f"记录操作: {action_type.value}, selector={selector}")
    
    def clear(self):
        """清除录制记录"""
        self.actions.clear()
        logger.info("录制记录已清除")


class ActionPlayer:
    """
    操作播放器
    回放录制的操作
    """
    
    def __init__(self, page: Page):
        self.page = page
    
    async def play(
        self,
        actions: List[RecordedAction],
        speed: float = 1.0,
        on_action: Optional[Callable[[RecordedAction], None]] = None,
    ):
        """
        播放操作
        
        Args:
            actions: 操作列表
            speed: 播放速度 (1.0 = 正常, 2.0 = 2倍速)
            on_action: 每个操作执行前的回调
        """
        logger.info(f"开始播放: {len(actions)} 个操作, 速度={speed}x")
        
        prev_timestamp = None
        
        for action in actions:
            # 计算等待时间
            if prev_timestamp is not None:
                wait_time = (action.timestamp - prev_timestamp) / speed
                if wait_time > 0:
                    await asyncio.sleep(wait_time / 1000)  # 转换为秒
            
            prev_timestamp = action.timestamp
            
            if on_action:
                on_action(action)
            
            await self._execute_action(action)
        
        logger.info("播放完成")
    
    async def _execute_action(self, action: RecordedAction):
        """执行单个操作"""
        try:
            if action.action_type == ActionType.CLICK:
                await self.page.click(action.selector)
                logger.debug(f"点击: {action.selector}")
            
            elif action.action_type == ActionType.FILL:
                await self.page.fill(action.selector, action.value)
                logger.debug(f"填充: {action.selector} = {action.value}")
            
            elif action.action_type == ActionType.TYPE:
                await self.page.type(action.selector, action.value)
                logger.debug(f"输入: {action.selector}")
            
            elif action.action_type == ActionType.SELECT:
                await self.page.select_option(action.selector, action.value)
                logger.debug(f"选择: {action.selector} = {action.value}")
            
            elif action.action_type == ActionType.CHECK:
                await self.page.check(action.selector)
                logger.debug(f"勾选: {action.selector}")
            
            elif action.action_type == ActionType.NAVIGATE:
                await self.page.goto(action.value)
                logger.debug(f"导航: {action.value}")
            
            elif action.action_type == ActionType.WAIT:
                await self.page.wait_for_timeout(float(action.value or 1000))
                logger.debug(f"等待: {action.value}ms")
            
            elif action.action_type == ActionType.HOVER:
                await self.page.hover(action.selector)
                logger.debug(f"悬停: {action.selector}")
            
            elif action.action_type == ActionType.KEYBOARD:
                await self.page.keyboard.press(action.key)
                logger.debug(f"按键: {action.key}")
            
        except Exception as e:
            logger.error(f"执行操作失败: {action.action_type.value}, error={e}")


class CodeGenerator:
    """
    代码生成器
    将录制的操作转换为代码
    """
    
    @staticmethod
    def generate_python(
        actions: List[RecordedAction],
        class_name: str = "TestRecorded",
        use_async: bool = True,
    ) -> str:
        """
        生成Python代码
        
        Args:
            actions: 操作列表
            class_name: 类名
            use_async: 是否使用异步
            
        Returns:
            str: 生成的代码
        """
        code = '''"""
自动生成的测试代码
生成时间: {timestamp}
"""
import asyncio
from playwright.async_api import async_playwright


class {class_name}:
    """录制的测试"""
    
    async def run(self):
        """执行录制操作"""
        async with async_playwright() as p:
            browser = await p.chromium.launch(headless=False)
            page = await browser.new_page()
            
            try:
{actions_code}
            finally:
                await browser.close()


if __name__ == "__main__":
    asyncio.run({class_name}().run())
'''
        
        actions_code = ""
        indent = "                "
        
        for i, action in enumerate(actions):
            if action.action_type == ActionType.CLICK:
                actions_code += f"{indent}# 步骤 {i+1}: 点击\n"
                actions_code += f"{indent}await page.click('{action.selector}')\n\n"
            
            elif action.action_type == ActionType.FILL:
                actions_code += f"{indent}# 步骤 {i+1}: 填充\n"
                actions_code += f"{indent}await page.fill('{action.selector}', '{action.value}')\n\n"
            
            elif action.action_type == ActionType.TYPE:
                actions_code += f"{indent}# 步骤 {i+1}: 输入\n"
                actions_code += f"{indent}await page.type('{action.selector}', '{action.value}')\n\n"
            
            elif action.action_type == ActionType.SELECT:
                actions_code += f"{indent}# 步骤 {i+1}: 选择\n"
                actions_code += f"{indent}await page.select_option('{action.selector}', '{action.value}')\n\n"
            
            elif action.action_type == ActionType.NAVIGATE:
                actions_code += f"{indent}# 步骤 {i+1}: 导航\n"
                actions_code += f"{indent}await page.goto('{action.value}')\n\n"
            
            elif action.action_type == ActionType.WAIT:
                actions_code += f"{indent}# 步骤 {i+1}: 等待\n"
                actions_code += f"{indent}await page.wait_for_timeout({action.value})\n\n"
        
        return code.format(
            timestamp=datetime.now().isoformat(),
            class_name=class_name,
            actions_code=actions_code,
        )
    
    @staticmethod
    def generate_playwright_test(
        actions: List[RecordedAction],
        test_name: str = "recorded_test",
    ) -> str:
        """
        生成Playwright测试代码
        
        Args:
            actions: 操作列表
            test_name: 测试名
            
        Returns:
            str: 生成的代码
        """
        code = '''/*
自动生成的Playwright测试
生成时间: {timestamp}
*/
import {{ test, expect }} from '@playwright/test';

test('{test_name}', async ({{ page }}) => {{
{actions_code}
}});
'''
        
        actions_code = ""
        indent = "  "
        
        for i, action in enumerate(actions):
            if action.action_type == ActionType.CLICK:
                actions_code += f"{indent}// 步骤 {i+1}: 点击\n"
                actions_code += f"{indent}await page.click('{action.selector}');\n\n"
            
            elif action.action_type == ActionType.FILL:
                actions_code += f"{indent}// 步骤 {i+1}: 填充\n"
                actions_code += f"{indent}await page.fill('{action.selector}', '{action.value}');\n\n"
            
            elif action.action_type == ActionType.NAVIGATE:
                actions_code += f"{indent}// 步骤 {i+1}: 导航\n"
                actions_code += f"{indent}await page.goto('{action.value}');\n\n"
        
        return code.format(
            timestamp=datetime.now().isoformat(),
            test_name=test_name,
            actions_code=actions_code,
        )


class RecordingSession:
    """
    录制会话
    管理录制、保存、加载
    """
    
    def __init__(self, page: Page):
        self.page = page
        self.recorder = ActionRecorder(page)
        self.player = ActionPlayer(page)
        self.generator = CodeGenerator()
    
    async def start_recording(self):
        """开始录制"""
        await self.recorder.start()
    
    async def stop_recording(self) -> List[RecordedAction]:
        """停止录制"""
        return await self.recorder.stop()
    
    async def replay(self, speed: float = 1.0):
        """回放录制"""
        await self.player.play(self.recorder.actions, speed)
    
    def save(self, filepath: str):
        """保存录制"""
        data = {
            "actions": [a.to_dict() for a in self.recorder.actions],
            "saved_at": datetime.now().isoformat(),
        }
        
        Path(filepath).parent.mkdir(parents=True, exist_ok=True)
        with open(filepath, 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=2)
        
        logger.info(f"录制已保存: {filepath}")
    
    def load(self, filepath: str) -> List[RecordedAction]:
        """加载录制"""
        with open(filepath, 'r', encoding='utf-8') as f:
            data = json.load(f)
        
        self.recorder.actions = [
            RecordedAction.from_dict(a) for a in data["actions"]
        ]
        
        logger.info(f"录制已加载: {filepath}, {len(self.recorder.actions)} 个操作")
        return self.recorder.actions
    
    def export_python(self, filepath: str, class_name: str = "TestRecorded"):
        """导出为Python代码"""
        code = self.generator.generate_python(
            self.recorder.actions,
            class_name,
        )
        
        Path(filepath).write_text(code, encoding='utf-8')
        logger.info(f"Python代码已导出: {filepath}")
    
    def export_playwright_test(self, filepath: str, test_name: str = "recorded_test"):
        """导出为Playwright测试"""
        code = self.generator.generate_playwright_test(
            self.recorder.actions,
            test_name,
        )
        
        Path(filepath).write_text(code, encoding='utf-8')
        logger.info(f"Playwright测试已导出: {filepath}")


"""
引擎子模块
提供规则引擎、LLM 引擎和混合引擎
"""

from .base import BaseEngine
from .rule_engine import RuleEngine
from .hybrid_engine import HybridEngine

__all__ = [
    "BaseEngine",
    "RuleEngine",
    "HybridEngine",
]



"""
AI 辅助模块类型定义
"""
from dataclasses import dataclass, field
from enum import Enum
from typing import List, Optional, Callable


class LLMProvider(Enum):
    """LLM 提供商"""
    RULE = "rule"           # 纯规则引擎
    CLAUDE = "claude"       # Anthropic Claude
    OPENAI = "openai"       # OpenAI GPT
    OLLAMA = "ollama"       # Ollama 本地模型
    CUSTOM = "custom"       # 自定义模型


@dataclass
class LLMConfig:
    """LLM 配置"""
    provider: LLMProvider = LLMProvider.RULE

    # Claude 配置
    claude_api_key: Optional[str] = None
    claude_model: str = "claude-sonnet-4-20250514"

    # OpenAI 配置
    openai_api_key: Optional[str] = None
    openai_model: str = "gpt-4o"
    openai_base_url: Optional[str] = None

    # Ollama 配置
    ollama_base_url: str = "http://localhost:11434"
    ollama_model: str = "llama3"

    # 自定义配置
    custom_call_fn: Optional[Callable[[str], str]] = None

    # 通用配置
    temperature: float = 0.7
    max_tokens: int = 1024
    timeout: int = 30


@dataclass
class AIConfig:
    """AI 配置"""
    enable_healing: bool = True
    enable_suggestions: bool = True
    max_suggestions: int = 5
    context_window: int = 1000

    # LLM 配置
    llm: LLMConfig = field(default_factory=LLMConfig)

    # 混合模式配置
    use_llm_first: bool = True          # 优先使用 LLM
    fallback_to_rule: bool = True       # LLM 失败时回退到规则引擎


@dataclass
class SelectorSuggestion:
    """选择器建议"""
    selector: str
    confidence: float
    strategy: str
    element_type: str
    description: str = ""


@dataclass
class ErrorDiagnosis:
    """错误诊断结果"""
    error_type: str
    likely_causes: List[str]
    suggested_solutions: List[str]
    related_elements: List[str] = field(default_factory=list)


@dataclass
class CodeSuggestion:
    """代码建议"""
    code: str
    description: str
    language: str = "python"
    confidence: float = 0.8



"""
诊断辅助类
整合 diagnostics 子模块，提供统一访问入口
"""
from typing import Callable, Optional
from playwright.async_api import Page

from ..diagnostics import (
    ConsoleMonitor,
    ConsoleMonitorConfig,
    ErrorAnalyzer,
    PageSnapshot,
    SnapshotConfig,
)


class DiagnosticsHelper:
    """
    诊断辅助类
    整合 diagnostics 子模块

    使用方式:
        browser.diagnostics.console  # ConsoleMonitor
        browser.diagnostics.error    # ErrorAnalyzer
        browser.diagnostics.snapshot  # PageSnapshot
    """

    def __init__(self, get_page: Callable[[], Page]):
        """
        初始化诊断辅助类

        Args:
            get_page: 获取当前 Page 对象的回调函数
        """
        self._get_page = get_page
        self._console: Optional[ConsoleMonitor] = None
        self._error: Optional[ErrorAnalyzer] = None
        self._snapshot: Optional[PageSnapshot] = None
        self._console_config: Optional[ConsoleMonitorConfig] = None
        self._snapshot_config: Optional[SnapshotConfig] = None

    @property
    def console(self) -> ConsoleMonitor:
        """
        获取控制台监控器

        Returns:
            ConsoleMonitor: 控制台监控器实例
        """
        if self._console is None:
            self._console = ConsoleMonitor(
                self._get_page(),
                self._console_config
            )
        return self._console

    @property
    def error(self) -> ErrorAnalyzer:
        """
        获取错误分析器

        Returns:
            ErrorAnalyzer: 错误分析器实例
        """
        if self._error is None:
            self._error = ErrorAnalyzer(self._get_page())
        return self._error

    @property
    def snapshot(self) -> PageSnapshot:
        """
        获取页面快照工具

        Returns:
            PageSnapshot: 页面快照实例
        """
        if self._snapshot is None:
            self._snapshot = PageSnapshot(
                self._get_page(),
                self._snapshot_config
            )
        return self._snapshot

    def set_console_config(self, config: ConsoleMonitorConfig):
        """
        设置控制台监控配置

        Args:
            config: 控制台监控配置
        """
        self._console_config = config
        # 重置实例以应用新配置
        self._console = None

    def set_snapshot_config(self, config: SnapshotConfig):
        """
        设置快照配置

        Args:
            config: 快照配置
        """
        self._snapshot_config = config
        self._snapshot = None

    def reset(self):
        """
        重置所有实例
        当切换标签页时调用，确保使用正确的 Page 对象
        """
        self._console = None
        self._error = None
        self._snapshot = None



"""
测试隔离器
支持独立测试环境、状态隔离、自动清理
"""
import asyncio
import uuid
import json
import shutil
from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any, Callable, Awaitable
from pathlib import Path
from datetime import datetime
from contextlib import asynccontextmanager
from loguru import logger


@dataclass
class IsolationConfig:
    """隔离配置"""
    workspace_dir: str = "./test_isolation"
    cleanup_on_exit: bool = True
    preserve_on_failure: bool = True
    snapshot_state: bool = True
    isolate_cookies: bool = True
    isolate_storage: bool = True
    isolate_network: bool = False


@dataclass
class IsolationContext:
    """隔离上下文"""
    id: str
    workspace: str
    created_at: datetime = field(default_factory=datetime.now)
    test_name: Optional[str] = None
    parent_context: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    cleanup_tasks: List[Callable[[], Awaitable[None]]] = field(default_factory=list)

    # 隔离状态
    cookies_backup: Optional[List[Dict]] = None
    storage_backup: Optional[Dict[str, Any]] = None
    network_requests: List[Dict] = field(default_factory=list)


@dataclass
class IsolationResult:
    """隔离结果"""
    success: bool
    context_id: str
    test_name: Optional[str] = None
    error: Optional[str] = None
    duration_seconds: float = 0.0
    cleanup_performed: bool = False
    artifacts_preserved: bool = False


class TestIsolator:
    """
    测试隔离器
    提供测试环境和状态的隔离
    """

    def __init__(
        self,
        page,
        config: Optional[IsolationConfig] = None,
    ):
        self.page = page
        self.config = config or IsolationConfig()
        self._contexts: Dict[str, IsolationContext] = {}
        self._current_context: Optional[IsolationContext] = None
        self._counter = 0

    # ==================== 上下文管理 ====================

    async def create_context(
        self,
        test_name: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> IsolationContext:
        """
        创建隔离上下文

        Args:
            test_name: 测试名称
            metadata: 元数据

        Returns:
            IsolationContext: 隔离上下文
        """
        self._counter += 1
        context_id = f"iso_{self._counter}_{uuid.uuid4().hex[:6]}"

        # 创建工作空间
        workspace = Path(self.config.workspace_dir) / context_id
        workspace.mkdir(parents=True, exist_ok=True)

        context = IsolationContext(
            id=context_id,
            workspace=str(workspace),
            test_name=test_name,
            metadata=metadata or {},
        )

        # 备份状态
        if self.config.snapshot_state:
            await self._backup_state(context)

        self._contexts[context_id] = context
        self._current_context = context

        logger.debug(f"创建隔离上下文: {context_id}")
        return context

    async def enter_context(
        self,
        test_name: Optional[str] = None,
    ) -> IsolationContext:
        """
        进入隔离上下文

        Args:
            test_name: 测试名称

        Returns:
            IsolationContext: 隔离上下文
        """
        context = await self.create_context(test_name)

        # 应用隔离
        if self.config.isolate_cookies:
            await self._isolate_cookies(context)

        if self.config.isolate_storage:
            await self._isolate_storage(context)

        return context

    async def exit_context(
        self,
        context_id: str,
        success: bool = True,
    ) -> IsolationResult:
        """
        退出隔离上下文

        Args:
            context_id: 上下文 ID
            success: 是否成功

        Returns:
            IsolationResult: 隔离结果
        """
        context = self._contexts.get(context_id)
        if not context:
            return IsolationResult(
                success=False,
                context_id=context_id,
                error="上下文不存在",
            )

        result = IsolationResult(
            success=success,
            context_id=context_id,
            test_name=context.test_name,
        )

        # 执行清理
        if self.config.cleanup_on_exit:
            preserve = not success and self.config.preserve_on_failure
            await self._cleanup(context, preserve)
            result.cleanup_performed = True
            result.artifacts_preserved = preserve

        # 恢复状态
        if self.config.snapshot_state:
            await self._restore_state(context)

        if self._current_context == context:
            self._current_context = None

        return result

    @asynccontextmanager
    async def isolate(
        self,
        test_name: Optional[str] = None,
    ):
        """
        隔离上下文管理器

        Args:
            test_name: 测试名称

        Yields:
            IsolationContext: 隔离上下文
        """
        context = await self.enter_context(test_name)
        success = True
        error = None

        try:
            yield context
        except Exception as e:
            success = False
            error = str(e)
            raise
        finally:
            result = await self.exit_context(context.id, success)
            if error:
                logger.error(f"隔离测试失败: {test_name}, error={error}")

    # ==================== 状态备份/恢复 ====================

    async def _backup_state(self, context: IsolationContext):
        """备份当前状态"""
        try:
            # 备份 Cookies
            if self.config.isolate_cookies:
                context.cookies_backup = await self.page.context.cookies()

            # 备份本地存储
            if self.config.isolate_storage:
                storage = await self.page.evaluate("""
                    () => ({
                        localStorage: {...localStorage},
                        sessionStorage: {...sessionStorage},
                    })
                """)
                context.storage_backup = storage

        except Exception as e:
            logger.warning(f"状态备份失败: {e}")

    async def _restore_state(self, context: IsolationContext):
        """恢复状态"""
        try:
            # 恢复 Cookies
            if context.cookies_backup:
                await self.page.context.clear_cookies()
                await self.page.context.add_cookies(context.cookies_backup)

            # 恢复存储
            if context.storage_backup:
                await self.page.evaluate("""
                    (storage) => {
                        // 清除并恢复 localStorage
                        localStorage.clear();
                        Object.entries(storage.localStorage || {}).forEach(([k, v]) => {
                            localStorage.setItem(k, v);
                        });
                        // 清除并恢复 sessionStorage
                        sessionStorage.clear();
                        Object.entries(storage.sessionStorage || {}).forEach(([k, v]) => {
                            sessionStorage.setItem(k, v);
                        });
                    }
                """, context.storage_backup)

        except Exception as e:
            logger.warning(f"状态恢复失败: {e}")

    async def _isolate_cookies(self, context: IsolationContext):
        """隔离 Cookies"""
        await self.page.context.clear_cookies()

    async def _isolate_storage(self, context: IsolationContext):
        """隔离存储"""
        await self.page.evaluate("""
            () => {
                localStorage.clear();
                sessionStorage.clear();
            }
        """)

    # ==================== 清理机制 ====================

    async def _cleanup(
        self,
        context: IsolationContext,
        preserve_artifacts: bool = False,
    ):
        """执行清理"""
        # 执行注册的清理任务
        for task in context.cleanup_tasks:
            try:
                await task()
            except Exception as e:
                logger.warning(f"清理任务失败: {e}")

        # 清理工作空间
        if not preserve_artifacts:
            workspace = Path(context.workspace)
            if workspace.exists():
                try:
                    shutil.rmtree(workspace)
                except Exception as e:
                    logger.warning(f"清理工作空间失败: {e}")

    def register_cleanup(
        self,
        context_id: str,
        task: Callable[[], Awaitable[None]],
    ):
        """
        注册清理任务

        Args:
            context_id: 上下文 ID
            task: 清理任务
        """
        context = self._contexts.get(context_id)
        if context:
            context.cleanup_tasks.append(task)

    # ==================== 工作空间管理 ====================

    def get_workspace(self, context_id: str) -> Optional[str]:
        """获取工作空间路径"""
        context = self._contexts.get(context_id)
        return context.workspace if context else None

    async def save_artifact(
        self,
        context_id: str,
        name: str,
        content: bytes,
    ) -> str:
        """
        保存工件

        Args:
            context_id: 上下文 ID
            name: 工件名称
            content: 工件内容

        Returns:
            str: 工件路径
        """
        context = self._contexts.get(context_id)
        if not context:
            raise ValueError(f"上下文不存在: {context_id}")

        artifacts_dir = Path(context.workspace) / "artifacts"
        artifacts_dir.mkdir(exist_ok=True)

        artifact_path = artifacts_dir / name
        with open(artifact_path, 'wb') as f:
            f.write(content)

        return str(artifact_path)

    async def save_screenshot(
        self,
        context_id: str,
        name: str,
    ) -> str:
        """保存截图"""
        context = self._contexts.get(context_id)
        if not context:
            raise ValueError(f"上下文不存在: {context_id}")

        screenshots_dir = Path(context.workspace) / "screenshots"
        screenshots_dir.mkdir(exist_ok=True)

        path = screenshots_dir / f"{name}.png"
        await self.page.screenshot(path=str(path))

        return str(path)

    # ==================== 信息查询 ====================

    def get_context(self, context_id: str) -> Optional[IsolationContext]:
        """获取上下文"""
        return self._contexts.get(context_id)

    def list_contexts(self) -> List[str]:
        """列出所有上下文"""
        return list(self._contexts.keys())

    def get_current_context(self) -> Optional[IsolationContext]:
        """获取当前上下文"""
        return self._current_context

    async def cleanup_all(self):
        """清理所有上下文"""
        for context_id in list(self._contexts.keys()):
            await self.exit_context(context_id, success=True)

        # 清理工作空间根目录
        root = Path(self.config.workspace_dir)
        if root.exists() and self.config.cleanup_on_exit:
            try:
                shutil.rmtree(root)
            except Exception as e:
                logger.warning(f"清理工作空间根目录失败: {e}")


def create_test_isolator(
    page,
    config: Optional[IsolationConfig] = None,
) -> TestIsolator:
    """创建测试隔离器实例"""
    return TestIsolator(page, config)



"""
验证器和检查器
提供各种验证功能
"""
import re
from typing import Optional, List, Dict, Any
from playwright.async_api import Page
from loguru import logger


class PageValidator:
    """
    页面验证器
    验证页面状态和内容
    """
    
    def __init__(self, page: Page):
        self.page = page
    
    async def validate_url(self, expected: str, exact: bool = True) -> bool:
        """验证URL"""
        actual = self.page.url
        if exact:
            result = actual == expected
        else:
            result = expected in actual
        
        if result:
            logger.debug(f"URL验证通过: {actual}")
        else:
            logger.error(f"URL验证失败: 期望 '{expected}', 实际 '{actual}'")
        
        return result
    
    async def validate_title(self, expected: str, contains: bool = True) -> bool:
        """验证标题"""
        actual = await self.page.title()
        
        if contains:
            result = expected in actual
        else:
            result = actual == expected
        
        if result:
            logger.debug(f"标题验证通过: {actual}")
        else:
            logger.error(f"标题验证失败: 期望 '{expected}', 实际 '{actual}'")
        
        return result
    
    async def validate_element_exists(self, selector: str) -> bool:
        """验证元素存在"""
        count = await self.page.locator(selector).count()
        result = count > 0
        
        if result:
            logger.debug(f"元素存在验证通过: {selector}")
        else:
            logger.error(f"元素不存在: {selector}")
        
        return result
    
    async def validate_element_visible(self, selector: str) -> bool:
        """验证元素可见"""
        try:
            visible = await self.page.locator(selector).is_visible()
            if visible:
                logger.debug(f"元素可见验证通过: {selector}")
            else:
                logger.error(f"元素不可见: {selector}")
            return visible
        except:
            logger.error(f"元素不存在或不可见: {selector}")
            return False
    
    async def validate_text_present(self, text: str) -> bool:
        """验证文本存在"""
        try:
            element = self.page.get_by_text(text)
            count = await element.count()
            result = count > 0
            
            if result:
                logger.debug(f"文本存在验证通过: {text[:50]}")
            else:
                logger.error(f"文本不存在: {text[:50]}")
            
            return result
        except:
            return False
    
    async def validate_text_in_element(self, selector: str, text: str) -> bool:
        """验证元素包含文本"""
        try:
            element_text = await self.page.locator(selector).text_content()
            result = text in (element_text or "")
            
            if result:
                logger.debug(f"元素文本验证通过: {selector}")
            else:
                logger.error(f"元素文本验证失败: {selector}")
            
            return result
        except:
            return False
    
    async def validate_image_loaded(self, selector: str) -> bool:
        """验证图片已加载"""
        try:
            complete = await self.page.locator(selector).evaluate(
                "el => el.complete && el.naturalHeight > 0"
            )
            
            if complete:
                logger.debug(f"图片加载验证通过: {selector}")
            else:
                logger.error(f"图片未加载: {selector}")
            
            return complete
        except:
            return False
    
    async def validate_link_working(self, selector: str) -> bool:
        """验证链接有效"""
        try:
            href = await self.page.locator(selector).get_attribute("href")
            if not href:
                logger.error(f"链接没有href: {selector}")
                return False
            
            # 检查链接格式
            if href.startswith("http"):
                logger.debug(f"链接格式有效: {href}")
                return True
            elif href.startswith("/") or href.startswith("#"):
                logger.debug(f"相对链接有效: {href}")
                return True
            
            return False
        except:
            return False
    
    async def validate_form_valid(self, form_selector: str = "form") -> Dict[str, Any]:
        """验证表单有效性"""
        result = {
            "valid": True,
            "errors": [],
        }
        
        try:
            # 检查必填字段
            required_fields = await self.page.locator(f"{form_selector} [required]").all()
            
            for field in required_fields:
                value = await field.input_value() if await field.evaluate("el => el.tagName") in ["INPUT", "TEXTAREA"] else ""
                
                if not value:
                    name = await field.get_attribute("name") or await field.get_attribute("id")
                    result["errors"].append(f"必填字段为空: {name}")
                    result["valid"] = False
            
            # 检查email格式
            email_fields = await self.page.locator(f"{form_selector} input[type='email']").all()
            email_pattern = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$'
            
            for field in email_fields:
                value = await field.input_value()
                if value and not re.match(email_pattern, value):
                    result["errors"].append(f"Email格式无效: {value}")
                    result["valid"] = False
            
        except Exception as e:
            result["valid"] = False
            result["errors"].append(str(e))
        
        return result
    
    async def validate_no_console_errors(self) -> List[str]:
        """验证没有控制台错误"""
        errors = []
        
        def handle_console(msg):
            if msg.type == "error":
                errors.append(msg.text)
        
        self.page.on("console", handle_console)
        
        # 等待一段时间收集错误
        import asyncio
        await asyncio.sleep(1)
        
        if errors:
            logger.error(f"发现控制台错误: {errors}")
        else:
            logger.debug("没有控制台错误")
        
        return errors
    
    async def validate_page_performance(self, thresholds: Dict[str, float] = None) -> Dict[str, Any]:
        """验证页面性能"""
        thresholds = thresholds or {
            "load_time": 3000,  # 3秒
            "dom_content_loaded": 2000,
        }
        
        metrics = await self.page.evaluate("""
            () => ({
                load_time: performance.timing.loadEventEnd - performance.timing.navigationStart,
                dom_content_loaded: performance.timing.domContentLoadedEventEnd - performance.timing.navigationStart,
            })
        """)
        
        result = {
            "passed": True,
            "metrics": metrics,
            "violations": [],
        }
        
        for key, threshold in thresholds.items():
            if metrics.get(key, 0) > threshold:
                result["violations"].append(f"{key}: {metrics[key]}ms > {threshold}ms")
                result["passed"] = False
        
        return result


class ContentChecker:
    """
    内容检查器
    检查页面内容
    """
    
    def __init__(self, page: Page):
        self.page = page
    
    async def check_spelling(self, selector: str = "body") -> List[str]:
        """检查拼写（简单实现）"""
        text = await self.page.locator(selector).text_content()
        
        # 常见拼写错误（示例）
        common_errors = {
            "teh": "the",
            "recieve": "receive",
            "occured": "occurred",
        }
        
        errors = []
        text_lower = text.lower()
        
        for wrong, correct in common_errors.items():
            if wrong in text_lower:
                errors.append(f"'{wrong}' 应该是 '{correct}'")
        
        return errors
    
    async def check_broken_links(self) -> List[Dict[str, str]]:
        """检查断链"""
        links = await self.page.locator("a[href]").all()
        broken = []
        
        for link in links[:20]:  # 限制检查数量
            try:
                href = await link.get_attribute("href")
                text = await link.text_content()
                
                if href and href.startswith("http"):
                    # 这里可以实际请求检查，但简化处理
                    pass
                    
            except Exception as e:
                broken.append({
                    "href": href,
                    "text": text,
                    "error": str(e),
                })
        
        return broken
    
    async def check_missing_alt_text(self) -> List[str]:
        """检查缺少alt的图片"""
        images = await self.page.locator("img:not([alt])").all()
        return [await img.get_attribute("src") or "unknown" for img in images]
    
    async def check_empty_links(self) -> List[str]:
        """检查空链接"""
        links = await self.page.locator("a[href=''], a:not([href])").all()
        return [await link.text_content() or "empty" for link in links]
    
    async def check_duplicate_ids(self) -> List[str]:
        """检查重复ID"""
        ids = await self.page.evaluate("""
            () => {
                const ids = {};
                const duplicates = [];
                document.querySelectorAll('[id]').forEach(el => {
                    if (ids[el.id]) {
                        duplicates.push(el.id);
                    } else {
                        ids[el.id] = true;
                    }
                });
                return duplicates;
            }
        """)
        
        return ids


class SecurityChecker:
    """
    安全检查器
    检查页面安全性
    """
    
    def __init__(self, page: Page):
        self.page = page
    
    async def check_https(self) -> bool:
        """检查HTTPS"""
        url = self.page.url
        result = url.startswith("https://")
        
        if result:
            logger.debug("使用HTTPS")
        else:
            logger.warning("未使用HTTPS")
        
        return result
    
    async def check_mixed_content(self) -> List[str]:
        """检查混合内容"""
        mixed = await self.page.evaluate("""
            () => {
                const issues = [];
                
                // HTTP资源在HTTPS页面
                if (location.protocol === 'https:') {
                    document.querySelectorAll('img[src^="http:"], script[src^="http:"], link[href^="http:"]').forEach(el => {
                        issues.push(el.src || el.href);
                    });
                }
                
                return issues;
            }
        """)
        
        return mixed
    
    async def check_xss_vulnerabilities(self) -> List[str]:
        """检查XSS漏洞（基本）"""
        vulnerabilities = []
        
        # 检查是否有直接innerHTML赋值
        # 这只是基本检查，不是完整的XSS扫描
        
        return vulnerabilities
    
    async def check_cookies_secure(self) -> List[Dict[str, Any]]:
        """检查Cookie安全性"""
        cookies = await self.page.context.cookies()
        insecure = []
        
        for cookie in cookies:
            issues = []
            
            if not cookie.get("secure"):
                issues.append("未设置Secure标志")
            
            if not cookie.get("httpOnly"):
                issues.append("未设置HttpOnly标志")
            
            if issues:
                insecure.append({
                    "name": cookie.get("name"),
                    "issues": issues,
                })
        
        return insecure



"""
浏览器扩展支持
支持Chrome扩展的加载和管理
"""
import json
from typing import Optional, List, Dict, Any
from pathlib import Path
from playwright.async_api import BrowserContext
from loguru import logger


class ExtensionManager:
    """
    浏览器扩展管理器
    支持加载、启用、禁用Chrome扩展
    """
    
    def __init__(self, context: BrowserContext):
        self.context = context
        self._extensions: Dict[str, Dict] = {}
    
    async def load_from_path(self, extension_path: str) -> Optional[str]:
        """
        从路径加载扩展
        
        Args:
            extension_path: 扩展目录路径
            
        Returns:
            Optional[str]: 扩展ID
        """
        path = Path(extension_path)
        
        if not path.exists():
            logger.error(f"扩展路径不存在: {extension_path}")
            return None
        
        # 检查manifest.json
        manifest_path = path / "manifest.json"
        if not manifest_path.exists():
            logger.error(f"缺少manifest.json: {extension_path}")
            return None
        
        with open(manifest_path, 'r', encoding='utf-8') as f:
            manifest = json.load(f)
        
        extension_id = self._generate_extension_id(path)
        
        self._extensions[extension_id] = {
            "path": extension_path,
            "name": manifest.get("name", "Unknown"),
            "version": manifest.get("version", "1.0"),
            "manifest": manifest,
        }
        
        logger.info(f"加载扩展: {manifest.get('name')} ({extension_id})")
        return extension_id
    
    def _generate_extension_id(self, path: Path) -> str:
        """生成扩展ID"""
        import hashlib
        return hashlib.md5(str(path).encode()).hexdigest()[:32]
    
    async def load_from_crx(self, crx_path: str) -> Optional[str]:
        """
        从.crx文件加载扩展
        
        Args:
            crx_path: .crx文件路径
            
        Returns:
            Optional[str]: 扩展ID
        """
        # 解压crx文件
        import zipfile
        import tempfile
        
        crx_file = Path(crx_path)
        if not crx_file.exists():
            logger.error(f"CRX文件不存在: {crx_path}")
            return None
        
        # 创建临时目录解压
        temp_dir = Path(tempfile.mkdtemp())
        
        try:
            # CRX文件实际上是特殊格式的ZIP
            with zipfile.ZipFile(crx_file, 'r') as zf:
                zf.extractall(temp_dir)
            
            return await self.load_from_path(str(temp_dir))
            
        except Exception as e:
            logger.error(f"解压CRX文件失败: {e}")
            return None
    
    def list_extensions(self) -> List[Dict[str, Any]]:
        """列出所有已加载扩展"""
        return [
            {
                "id": ext_id,
                "name": ext["name"],
                "version": ext["version"],
                "path": ext["path"],
            }
            for ext_id, ext in self._extensions.items()
        ]
    
    def get_extension(self, extension_id: str) -> Optional[Dict]:
        """获取扩展信息"""
        return self._extensions.get(extension_id)
    
    def remove_extension(self, extension_id: str):
        """移除扩展"""
        if extension_id in self._extensions:
            del self._extensions[extension_id]
            logger.info(f"移除扩展: {extension_id}")


class AdBlocker:
    """
    广告拦截器
    拦截常见广告和追踪器
    """
    
    # 常见广告域名
    AD_DOMAINS = [
        "doubleclick.net",
        "googlesyndication.com",
        "googleadservices.com",
        "facebook.com/tr",
        "analytics.google.com",
        "google-analytics.com",
        "adservice.google.com",
        "ad.doubleclick.net",
        "pagead2.googlesyndication.com",
        "ads.twitter.com",
        "ads.linkedin.com",
    ]
    
    # 追踪器域名
    TRACKER_DOMAINS = [
        "facebook.net",
        "connect.facebook.net",
        "platform.twitter.com",
        "staticxx.facebook.com",
        "analytics.twitter.com",
        "pixel.facebook.com",
    ]
    
    def __init__(self, page):
        self.page = page
        self._blocked_count = 0
    
    async def enable(self):
        """启用广告拦截"""
        all_domains = self.AD_DOMAINS + self.TRACKER_DOMAINS
        
        async def block_ads(route):
            url = route.request.url
            
            for domain in all_domains:
                if domain in url:
                    self._blocked_count += 1
                    logger.debug(f"拦截广告/追踪器: {url}")
                    await route.abort()
                    return
            
            await route.continue_()
        
        await self.page.route("**/*", block_ads)
        logger.info("广告拦截已启用")
    
    async def disable(self):
        """禁用广告拦截"""
        await self.page.unroute("**/*")
        logger.info("广告拦截已禁用")
    
    def get_blocked_count(self) -> int:
        """获取拦截数量"""
        return self._blocked_count


class PrivacyProtector:
    """
    隐私保护器
    保护用户隐私，防止追踪
    """
    
    def __init__(self, page):
        self.page = page
    
    async def block_webrtc(self):
        """阻止WebRTC泄露"""
        await self.page.add_init_script("""
            // 阻止WebRTC泄露真实IP
            const RTC = window.RTCPeerConnection || window.webkitRTCPeerConnection;
            if (RTC) {
                window.RTCPeerConnection = function(...args) {
                    return new RTC(...args);
                };
                window.RTCPeerConnection.prototype = RTC.prototype;
            }
        """)
        logger.info("WebRTC泄露保护已启用")
    
    async def block_canvas_fingerprint(self):
        """阻止Canvas指纹"""
        await self.page.add_init_script("""
            // 添加噪声到Canvas指纹
            const originalToDataURL = HTMLCanvasElement.prototype.toDataURL;
            HTMLCanvasElement.prototype.toDataURL = function(type) {
                if (type === 'image/png' && this.width === 280 && this.height === 60) {
                    // 可能是指纹检测
                    const context = this.getContext('2d');
                    const imageData = context.getImageData(0, 0, this.width, this.height);
                    for (let i = 0; i < imageData.data.length; i += 4) {
                        imageData.data[i] ^= (Math.random() * 2) | 0;
                    }
                    context.putImageData(imageData, 0, 0);
                }
                return originalToDataURL.apply(this, arguments);
            };
        """)
        logger.info("Canvas指纹保护已启用")
    
    async def block_audio_fingerprint(self):
        """阻止音频指纹"""
        await self.page.add_init_script("""
            // 添加噪声到音频指纹
            const audioContext = window.AudioContext || window.webkitAudioContext;
            if (audioContext) {
                const originalCreateAnalyser = audioContext.prototype.createAnalyser;
                audioContext.prototype.createAnalyser = function() {
                    const analyser = originalCreateAnalyser.apply(this, arguments);
                    const originalGetFloatFrequencyData = analyser.getFloatFrequencyData;
                    analyser.getFloatFrequencyData = function(array) {
                        originalGetFloatFrequencyData.apply(this, arguments);
                        for (let i = 0; i < array.length; i++) {
                            array[i] += (Math.random() - 0.5) * 0.1;
                        }
                    };
                    return analyser;
                };
            }
        """)
        logger.info("音频指纹保护已启用")
    
    async def enable_all_protection(self):
        """启用所有隐私保护"""
        await self.block_webrtc()
        await self.block_canvas_fingerprint()
        await self.block_audio_fingerprint()
        logger.info("所有隐私保护已启用")
    
    async def clear_storage(self):
        """清除所有存储"""
        await self.page.evaluate("""
            () => {
                localStorage.clear();
                sessionStorage.clear();
                document.cookie.split(';').forEach(c => {
                    document.cookie = c.replace(/^ +/, '').replace(/=.*/, '=;expires=' + new Date().toUTCString() + ';path=/');
                });
            }
        """)
        logger.info("存储已清除")
    
    async def clear_cache(self):
        """清除缓存"""
        cdp = await self.page.context.new_cdp_session(self.page)
        await cdp.send('Network.clearBrowserCache')
        logger.info("缓存已清除")


class CookieConsentHandler:
    """
    Cookie同意处理
    自动处理Cookie同意弹窗
    """
    
    def __init__(self, page):
        self.page = page
    
    async def auto_accept(self):
        """自动接受Cookie同意"""
        accept_selectors = [
            # 通用选择器
            "button[id*='accept']",
            "button[id*='cookie']",
            "button[class*='accept']",
            "a[class*='accept']",
            "[data-testid*='accept']",
            "[data-testid*='cookie']",
            
            # 常见CMP
            "#onetrust-accept-btn-handler",
            ".cc-accept",
            ".cc-dismiss",
            "#ccc-notify-accept",
            "#cookie-banner-accept",
            "[aria-label*='Accept']",
            "[aria-label*='accept']",
            "[aria-label*='Akkoord']",  # 荷兰语
            "[aria-label*='Aceptar']",  # 西班牙语
            "[aria-label*='Accepter']", # 法语
            
            # 特定平台
            "button[data-testid*='cookie-banner-accept']",
            "button[data-cy*='accept-cookies']",
        ]
        
        for selector in accept_selectors:
            try:
                element = self.page.locator(selector)
                if await element.count() > 0:
                    await element.first.click(timeout=1000)
                    logger.info(f"已接受Cookie同意: {selector}")
                    return True
            except:
                continue
        
        logger.info("未找到Cookie同意弹窗")
        return False
    
    async def auto_reject(self):
        """自动拒绝Cookie同意"""
        reject_selectors = [
            "button[id*='reject']",
            "button[class*='reject']",
            "button[data-testid*='reject']",
            "[aria-label*='Reject']",
            "[aria-label*='拒绝']",
            "a[class*='reject']",
            ".cc-reject",
            "#ccc-notify-reject",
        ]
        
        for selector in reject_selectors:
            try:
                element = self.page.locator(selector)
                if await element.count() > 0:
                    await element.first.click(timeout=1000)
                    logger.info(f"已拒绝Cookie同意: {selector}")
                    return True
            except:
                continue
        
        # 如果找不到拒绝按钮，尝试接受
        return await self.auto_accept()


"""
请求限流器
支持令牌桶、漏桶、滑动窗口算法
"""
import asyncio
import time
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import Optional, Callable, List, Any, Literal
from loguru import logger


@dataclass
class RateLimiterConfig:
    """限流器配置"""
    requests_per_second: float = 10.0
    algorithm: Literal["token-bucket", "leaky-bucket", "sliding-window"] = "token-bucket"
    bucket_size: Optional[int] = None  # 仅用于令牌桶


@dataclass
class RateLimiterStats:
    """限流器统计"""
    total_requests: int = 0
    total_wait_time: float = 0.0  # 毫秒
    waiting: int = 0
    available: int = 0


class RateLimiterBase(ABC):
    """限流器基类"""

    def __init__(self, requests_per_second: float):
        self.requests_per_second = requests_per_second

    @abstractmethod
    async def acquire(self) -> None:
        """获取请求许可"""
        pass

    @abstractmethod
    def get_stats(self) -> dict:
        """获取状态"""
        pass


class TokenBucketLimiter(RateLimiterBase):
    """令牌桶限流器"""

    def __init__(self, requests_per_second: float, bucket_size: Optional[int] = None):
        super().__init__(requests_per_second)
        self.bucket_size = bucket_size or int(requests_per_second)
        self.tokens = float(self.bucket_size)
        self.last_refill_time = time.time()
        self._waiting_queue: List[asyncio.Event] = []
        self._lock = asyncio.Lock()

    async def acquire(self) -> None:
        """获取请求许可"""
        async with self._lock:
            self._refill()

            if self.tokens >= 1:
                self.tokens -= 1
                return

        # 需要等待
        event = asyncio.Event()
        self._waiting_queue.append(event)

        # 计算等待时间
        time_to_next_token = (1 - self.tokens) / self.requests_per_second
        await asyncio.sleep(time_to_next_token)

        async with self._lock:
            self._refill()
            if self.tokens >= 1:
                self.tokens -= 1
                event.set()
                if event in self._waiting_queue:
                    self._waiting_queue.remove(event)

    def _refill(self) -> None:
        """补充令牌"""
        now = time.time()
        elapsed = now - self.last_refill_time
        tokens_to_add = elapsed * self.requests_per_second

        self.tokens = min(self.bucket_size, self.tokens + tokens_to_add)
        self.last_refill_time = now

    def get_stats(self) -> dict:
        """获取状态"""
        self._refill()
        return {
            "waiting": len(self._waiting_queue),
            "available": int(self.tokens),
        }


class LeakyBucketLimiter(RateLimiterBase):
    """漏桶限流器"""

    def __init__(self, requests_per_second: float):
        super().__init__(requests_per_second)
        self._queue: List[asyncio.Event] = []
        self._is_processing = False
        self._lock = asyncio.Lock()

    async def acquire(self) -> None:
        """获取请求许可"""
        event = asyncio.Event()

        async with self._lock:
            self._queue.append(event)
            if not self._is_processing:
                asyncio.create_task(self._process_queue())

        await event.wait()

    async def _process_queue(self) -> None:
        """处理队列"""
        self._is_processing = True
        interval = 1.0 / self.requests_per_second

        while True:
            async with self._lock:
                if not self._queue:
                    self._is_processing = False
                    return

                event = self._queue.pop(0)

            event.set()
            await asyncio.sleep(interval)

    def get_stats(self) -> dict:
        """获取状态"""
        return {
            "waiting": len(self._queue),
            "available": 1 if not self._queue else 0,
        }


class SlidingWindowLimiter(RateLimiterBase):
    """滑动窗口限流器"""

    def __init__(self, requests_per_second: float):
        super().__init__(requests_per_second)
        self.window_size = 1.0  # 1秒窗口
        self._requests: List[float] = []
        self._lock = asyncio.Lock()

    async def acquire(self) -> None:
        """获取请求许可"""
        async with self._lock:
            now = time.time()
            self._clean_old_requests(now)

            max_requests = int(self.requests_per_second) + 1

            if len(self._requests) < max_requests:
                self._requests.append(now)
                return

        # 需要等待
        oldest_request = self._requests[0]
        wait_time = oldest_request + self.window_size - time.time()

        if wait_time > 0:
            await asyncio.sleep(wait_time)

        async with self._lock:
            now = time.time()
            self._clean_old_requests(now)
            self._requests.append(now)

    def _clean_old_requests(self, now: float) -> None:
        """清理过期请求"""
        cutoff = now - self.window_size
        self._requests = [t for t in self._requests if t > cutoff]

    def get_stats(self) -> dict:
        """获取状态"""
        self._clean_old_requests(time.time())
        max_requests = int(self.requests_per_second) + 1
        return {
            "waiting": 0,
            "available": max(0, max_requests - len(self._requests)),
        }


class RateLimiter:
    """
    请求限流器
    支持多种限流算法
    """

    def __init__(self, config: Optional[RateLimiterConfig] = None):
        self.config = config or RateLimiterConfig()
        self._limiter = self._create_limiter()
        self._total_requests = 0
        self._total_wait_time = 0.0

    def _create_limiter(self) -> RateLimiterBase:
        """创建限流器实例"""
        algorithm = self.config.algorithm

        if algorithm == "leaky-bucket":
            return LeakyBucketLimiter(self.config.requests_per_second)
        elif algorithm == "sliding-window":
            return SlidingWindowLimiter(self.config.requests_per_second)
        else:  # token-bucket
            return TokenBucketLimiter(
                self.config.requests_per_second,
                self.config.bucket_size,
            )

    async def acquire(self) -> None:
        """获取请求许可"""
        start_time = time.time()
        await self._limiter.acquire()

        self._total_requests += 1
        self._total_wait_time += (time.time() - start_time) * 1000

    async def execute(self, fn: Callable) -> Any:
        """
        执行带限流的函数

        Args:
            fn: 要执行的函数

        Returns:
            Any: 函数返回值
        """
        await self.acquire()
        if asyncio.iscoroutinefunction(fn):
            return await fn()
        else:
            return fn()

    async def execute_batch(
        self,
        tasks: List[Callable],
        on_progress: Optional[Callable[[int, int], None]] = None,
    ) -> List[Any]:
        """
        批量执行带限流

        Args:
            tasks: 任务列表
            on_progress: 进度回调

        Returns:
            List[Any]: 结果列表
        """
        results = []

        for i, task in enumerate(tasks):
            result = await self.execute(task)
            results.append(result)

            if on_progress:
                if asyncio.iscoroutinefunction(on_progress):
                    await on_progress(i + 1, len(tasks))
                else:
                    on_progress(i + 1, len(tasks))

        return results

    def get_stats(self) -> RateLimiterStats:
        """获取统计信息"""
        current_stats = self._limiter.get_stats()
        return RateLimiterStats(
            total_requests=self._total_requests,
            total_wait_time=self._total_wait_time,
            waiting=current_stats["waiting"],
            available=current_stats["available"],
        )

    def reset_stats(self) -> None:
        """重置统计"""
        self._total_requests = 0
        self._total_wait_time = 0.0

    def update_rate(self, requests_per_second: float) -> None:
        """更新请求速率"""
        self.config.requests_per_second = requests_per_second
        self._limiter = self._create_limiter()
        logger.debug(f"限流器速率更新: {requests_per_second} req/s")


def create_rate_limiter(config: Optional[RateLimiterConfig] = None) -> RateLimiter:
    """创建限流器实例"""
    return RateLimiter(config)



"""
选择器增强模块
支持多种选择器类型和链式定位
"""
from typing import Optional, Union, List
from playwright.async_api import Page, Locator
from loguru import logger
from enum import Enum


class SelectorType(Enum):
    """选择器类型"""
    CSS = "css"           # CSS选择器
    XPATH = "xpath"       # XPath
    TEXT = "text"         # 文本匹配
    ROLE = "role"         # ARIA角色
    PLACEHOLDER = "placeholder"  # 占位符
    LABEL = "label"       # 标签
    TEST_ID = "test-id"   # 测试ID
    ALT_TEXT = "alt-text" # 图片alt文本
    TITLE = "title"       # title属性


class SmartSelector:
    """
    智能选择器
    支持多种选择器类型和链式定位
    """
    
    def __init__(self, page: Page):
        self.page = page
    
    def css(self, selector: str) -> Locator:
        """CSS选择器"""
        return self.page.locator(selector)
    
    def xpath(self, selector: str) -> Locator:
        """XPath选择器"""
        return self.page.locator(f"xpath={selector}")
    
    def text(self, text: str, exact: bool = False) -> Locator:
        """
        文本选择器
        
        Args:
            text: 要匹配的文本
            exact: 是否精确匹配
        """
        return self.page.get_by_text(text, exact=exact)
    
    def role(self, role: str, name: Optional[str] = None) -> Locator:
        """
        ARIA角色选择器
        
        Args:
            role: ARIA角色 (button, link, textbox, etc.)
            name: 可访问名称
        """
        return self.page.get_by_role(role, name=name)
    
    def placeholder(self, text: str) -> Locator:
        """占位符选择器"""
        return self.page.get_by_placeholder(text)
    
    def label(self, text: str) -> Locator:
        """标签选择器"""
        return self.page.get_by_label(text)
    
    def test_id(self, test_id: str) -> Locator:
        """测试ID选择器"""
        return self.page.get_by_test_id(test_id)
    
    def alt_text(self, text: str) -> Locator:
        """图片alt文本选择器"""
        return self.page.get_by_alt_text(text)
    
    def title(self, text: str) -> Locator:
        """title属性选择器"""
        return self.page.get_by_title(text)
    
    def chain(self, *selectors: str) -> Locator:
        """
        链式选择器
        
        Args:
            *selectors: 选择器链
            
        Returns:
            Locator: 链式定位结果
            
        Example:
            selector.chain("div.container", "ul.list", "li.item")
            等价于: page.locator("div.container >> ul.list >> li.item")
        """
        chain_str = " >> ".join(selectors)
        return self.page.locator(chain_str)
    
    def filter(
        self, 
        parent: str, 
        child: str,
        has_text: Optional[str] = None,
        has_not_text: Optional[str] = None,
    ) -> Locator:
        """
        过滤选择器
        
        Args:
            parent: 父元素选择器
            child: 子元素选择器
            has_text: 必须包含的文本
            has_not_text: 不能包含的文本
        """
        locator = self.page.locator(parent).locator(child)
        
        if has_text:
            locator = locator.filter(has_text=has_text)
        if has_not_text:
            locator = locator.filter(has_not_text=has_not_text)
        
        return locator
    
    def nth(self, selector: str, index: int) -> Locator:
        """
        第N个元素
        
        Args:
            selector: 选择器
            index: 索引（从0开始）
        """
        return self.page.locator(selector).nth(index)
    
    def first(self, selector: str) -> Locator:
        """第一个匹配元素"""
        return self.page.locator(selector).first
    
    def last(self, selector: str) -> Locator:
        """最后一个匹配元素"""
        return self.page.locator(selector).last
    
    async def smart_find(self, hint: str) -> Optional[Locator]:
        """
        智能查找元素
        尝试多种方式查找元素

        Args:
            hint: 查找提示（文本、ID、class等）
        """
        strategies = [
            # 1. 精确文本匹配
            lambda: self.text(hint, exact=True),
            # 2. 模糊文本匹配
            lambda: self.text(hint),
            # 3. 角色匹配
            lambda: self.role("button", name=hint),
            lambda: self.role("link", name=hint),
            # 4. 占位符
            lambda: self.placeholder(hint),
            # 5. 标签
            lambda: self.label(hint),
            # 6. CSS选择器
            lambda: self.css(hint),
            # 7. XPath
            lambda: self.xpath(f"//*[contains(text(), '{hint}')]"),
        ]

        for strategy in strategies:
            try:
                locator = strategy()
                count = await locator.count()
                if count > 0:
                    logger.debug(f"智能查找成功: hint={hint}")
                    return locator.first
            except:
                continue

        logger.warning(f"智能查找失败: hint={hint}")
        return None
    
    async def all(self, selector: str) -> List[Locator]:
        """获取所有匹配的元素"""
        locator = self.page.locator(selector)
        count = await locator.count()
        return [locator.nth(i) for i in range(count)]

    async def count(self, selector: str) -> int:
        """统计匹配元素数量"""
        return await self.page.locator(selector).count()
